unit Circuit;

interface

uses Math387, CuBlasMtxVec, Classes, Components;

type

   TCircuit = Class
   private
   num_nodes, num_vsources, num_nodess,
   num_inductors, num_opamps, num_equations: Integer;
   firstpass, nonlinear: Boolean;
   Jac: TMtx;
   MNA: TVec;
   Cir: TList;
   procedure Eval(t: TSample; x, dxdt: TVec);
   public
   constructor Create;
   function getSize: integer;
   function getNum_Nodes: integer;
   function getNonlinearity: Boolean;
   procedure setNonlinearity(no_linear: boolean);
   procedure ShowSolution;
   procedure Add(Component: TElement);
   procedure DaeFunction(t:TSample;x,dxdt,result:TVec);
   procedure Jacobian(t: TSample; x, dxdt:TVec; Jac: TMtx);
   function DCAnalysis: TVec;
   destructor Destroy; override;
   end;



implementation

uses Math;

{ TDaeProblem }

constructor TCircuit.Create;
begin
  MNA:= TVec.Create;
  MNA.Size(0);
  Cir:= TList.Create;
  nonlinear:= false;
end;

procedure TCircuit.DaeFunction(t: TSample; x, dxdt, result: TVec);
begin
  Eval(t, x, dxdt);
  result.Assign(MNA);
end;


procedure TCircuit.Add(Component: TElement);
var nodes_element: integer;
begin
   Cir.Add(Component);
   nodes_element:= 0;
   if (TElement(Component) is TVSinSource) then
    begin
      nodes_element:= math.max(TVSinSource(Component).getp,TVSinSource(Component).getn);
      num_nodes:= math.Max(num_nodes, nodes_element);
      inc(num_vsources);
    end;
   if (TElement(Component) is TVABSSinSource) then
    begin
      nodes_element:= math.max(TVABSSinSource(Component).getp,TVABSSinSource(Component).getn);
      num_nodes:= math.Max(num_nodes, nodes_element);
      inc(num_vsources);
    end;
   if (TElement(Component) is TDCVSource) then
    begin
      nodes_element:= math.max(TDCVSource(Component).getp,TDCVSource(Component).getn);
      num_nodes:= math.Max(num_nodes, nodes_element);
      inc(num_vsources);
    end;
   if (TElement(Component) is TDCISource)then
    begin
      nodes_element:= math.max(TDCISource(Component).getp,TDCISource(Component).getn);
      num_nodes:= math.Max(num_nodes, nodes_element);
    end;
   if (TElement(Component) is TDiode) then
    begin
      nonlinear:= true;
      nodes_element:= math.max(TDiode(Component).getp,TDiode(Component).getn);
      num_nodes:= math.Max(num_nodes, nodes_element);
    end;
   if (TElement(Component) is TDiodeZener) then
    begin
      nonlinear:= true;
      nodes_element:= math.max(TDiodeZener(Component).getp,TDiodeZener(Component).getn);
      num_nodes:= math.Max(num_nodes, nodes_element);
    end;
   if (TElement(Component) is TDiodeModelica) then
    begin
      nonlinear:= true;
      nodes_element:= math.max(TDiodeModelica(Component).getp,TDiodeModelica(Component).getn);
      num_nodes:= math.Max(num_nodes, nodes_element);
    end;
   if (TElement(Component) is TResistor) then
    begin
      nodes_element:= math.max(TResistor(Component).getp,TResistor(Component).getn);
      num_nodes:= math.Max(num_nodes, nodes_element);
    end;
   if (TElement(Component) is TChuaResistor) then
    begin
      nonlinear:= true;
      nodes_element:= math.max(TChuaResistor(Component).getp,TChuaResistor(Component).getn);
      num_nodes:= math.Max(num_nodes, nodes_element);
    end;
   if (TElement(Component) is TBJTNPN) then
    begin
      nonlinear:= true;
      nodes_element:= math.max(TBJTNPN(Component).getb,TBJTNPN(Component).getc);
      nodes_element:= math.max(nodes_element,TBJTNPN(Component).gete);
      num_nodes:= math.Max(num_nodes, nodes_element);
    end;
   if (TElement(Component) is TBJTEMNPN) then
    begin
      nonlinear:= true;
      nodes_element:= math.max(TBJTEMNPN(Component).getb,TBJTEMNPN(Component).getc);
      nodes_element:= math.max(nodes_element,TBJTEMNPN(Component).gete);
      num_nodes:= math.Max(num_nodes, nodes_element);
    end;
   if (TElement(Component) is TBJTPWLNPN) then
    begin
      nonlinear:= true;
      nodes_element:= math.max(TBJTPWLNPN(Component).getb,TBJTPWLNPN(Component).getc);
      nodes_element:= math.max(nodes_element,TBJTPWLNPN(Component).gete);
      num_nodes:= math.Max(num_nodes, nodes_element);
    end;
   if (TElement(Component) is TBJTPWLPNP) then
    begin
      nonlinear:= true;
      nodes_element:= math.max(TBJTPWLPNP(Component).getb,TBJTPWLPNP(Component).getc);
      nodes_element:= math.max(nodes_element,TBJTPWLPNP(Component).gete);
      num_nodes:= math.Max(num_nodes, nodes_element);
    end;
   if (TElement(Component) is TBJTEMPNP) then
    begin
      nonlinear:= true;
      nodes_element:= math.max(TBJTEMPNP(Component).getb,TBJTEMPNP(Component).getc);
      nodes_element:= math.max(nodes_element,TBJTEMPNP(Component).gete);
      num_nodes:= math.Max(num_nodes, nodes_element);
    end;
   if (TElement(Component) is TBJTNPNF) then
    begin
      nonlinear:= true;
      nodes_element:= math.max(TBJTNPNF(Component).getb,TBJTNPNF(Component).getc);
      nodes_element:= math.max(nodes_element,TBJTNPNF(Component).gete);
      num_nodes:= math.Max(num_nodes, nodes_element);
    end;
    if (TElement(Component) is TBJTPNP) then
    begin
      nonlinear:= true;
      nodes_element:= math.max(TBJTPNP(Component).getb,TBJTPNP(Component).getc);
      nodes_element:= math.max(nodes_element,TBJTPNP(Component).gete);
      num_nodes:= math.Max(num_nodes, nodes_element);
    end;
   if (TElement(Component) is TBJTPNPF) then
    begin
      nonlinear:= true;
      nodes_element:= math.max(TBJTPNPF(Component).getb,TBJTPNPF(Component).getc);
      nodes_element:= math.max(nodes_element,TBJTPNPF(Component).gete);
      num_nodes:= math.Max(num_nodes, nodes_element);
    end;
   if (TElement(Component) is TCapacitor) then
    begin
      nodes_element:= math.max(TCapacitor(Component).getp,TCapacitor(Component).getn);
      num_nodes:= math.Max(num_nodes, nodes_element);
    end;
   if (TElement(Component) is TInductor) then
    begin
      nodes_element:= math.max(TInductor(Component).getp,TInductor(Component).getn);
      num_nodes:= math.Max(num_nodes, nodes_element);
      inc(num_inductors);
    end;
   if (TElement(Component) is TOPAMPIDEAL) then
    begin
      nodes_element:= math.max(TOPAMPIDEAL(Component).getinp,TOPAMPIDEAL(Component).getinn);
      nodes_element:= math.Max(TOPAMPIDEAL(Component).getout, nodes_element);
      num_nodes:= math.Max(num_nodes, nodes_element);
      inc(num_vsources);
    end;
   if (TElement(Component) is TOPAMP) then
    begin
      nonlinear:= true;
      nodes_element:= math.max(TOPAMP(Component).getinp,TOPAMP(Component).getinn);
      nodes_element:= math.Max(TOPAMP(Component).getout, nodes_element);
      num_nodes:= math.Max(num_nodes, nodes_element);
      inc(num_vsources);
    end;
   if (TElement(Component) is TOPAMP741) then
    begin
      nonlinear:= true;
      nodes_element:= math.max(TOPAMP741(Component).getinp,TOPAMP741(Component).getinn);
      nodes_element:= math.Max(TOPAMP741(Component).getout, nodes_element);
      nodes_element:= math.Max(TOPAMP741(Component).getvcc, nodes_element);
      nodes_element:= math.Max(TOPAMP741(Component).getvee, nodes_element);
      num_nodes:= math.Max(num_nodes, nodes_element);
      inc(num_opamps,21);
    end;
   if (TElement(Component) is TVCT) then
    begin
       nodes_element:= math.max(TVCT(Component).getinp,TVCT(Component).getinn);
       nodes_element:= math.Max(TVCT(Component).getoutp, nodes_element);
       nodes_element:= math.Max(TVCT(Component).getoutn, nodes_element);
    end;
   if (TElement(Component) is TVVT) then
    begin
       nodes_element:= math.max(TVVT(Component).getinp,TVVT(Component).getinn);
       nodes_element:= math.Max(TVVT(Component).getoutp, nodes_element);
       nodes_element:= math.Max(TVVT(Component).getoutn, nodes_element);
       inc(num_vsources);
    end;
  num_equations:=(num_nodes+1)+num_vsources+num_inductors;
  MNA.Resize(num_equations);
  firstpass:= True;
end;

destructor TCircuit.Destroy;
var i: integer;
begin
  MNA.Destroy;
  for i:=Cir.Count-1 downto 0 do
    Cir.Delete(i);
  Cir.Destroy;
  inherited;
end;

procedure TCircuit.Eval(t: TSample; x, dxdt: TVec);
var incnum_equations, i: integer;
begin
  i:= 0;
  MNA.SetZero;
  incnum_equations:= 0;
  i:= 0;
  while (i < Cir.Count) do
  begin
    TElement(Cir[i]).Eval(t,x,dxdt);
    if (TElement(Cir[i]) is TDCVSource) then
      begin
      inc(incnum_equations);
        if (TDCVSource(Cir[i]).getp <> Datum) then
        begin
           MNA[TDCVSource(Cir[i]).getp]:= MNA[TDCVSource(Cir[i]).getp]-x[num_nodes+incnum_equations];
           MNA[num_nodes+incnum_equations]:= x[TDCVSource(Cir[i]).getp];
        end;
        if (TDCVSource(Cir[i]).getn <> Datum) then
        begin
           MNA[TDCVSource(Cir[i]).getn]:= MNA[TDCVSource(Cir[i]).getn]+x[num_nodes+incnum_equations];
           MNA[num_nodes+incnum_equations]:= MNA[num_nodes+incnum_equations]-x[TDCVSource(Cir[i]).getn];
        end;
        MNA[num_nodes+incnum_equations]:= MNA[num_nodes+incnum_equations]-TDCVSource(Cir[i]).getV;
      end;
    if (TElement(Cir[i]) is TVSinSource) then
      begin
      inc(incnum_equations);
        if (TVSinSource(Cir[i]).getp <> Datum) then
        begin
           MNA[TVSinSource(Cir[i]).getp]:= MNA[TVSinSource(Cir[i]).getp]-x[num_nodes+incnum_equations];
           MNA[num_nodes+incnum_equations]:= x[TVSinSource(Cir[i]).getp];
        end;
        if (TVSinSource(Cir[i]).getn <> Datum) then
        begin
           MNA[TVSinSource(Cir[i]).getn]:= MNA[TVSinSource(Cir[i]).getn]+x[num_nodes+incnum_equations];
           MNA[num_nodes+incnum_equations]:= MNA[num_nodes+incnum_equations]-x[TVSinSource(Cir[i]).getn];
        end;
          MNA[num_nodes+incnum_equations]:= MNA[num_nodes+incnum_equations]-TVSinSource(Cir[i]).getV;
      end;
    if (TElement(Cir[i]) is TVABSSinSource) then
      begin
      inc(incnum_equations);
        if (TVABSSinSource(Cir[i]).getp <> Datum) then
        begin
           MNA[TVABSSinSource(Cir[i]).getp]:= MNA[TVABSSinSource(Cir[i]).getp]-x[num_nodes+incnum_equations];
           MNA[num_nodes+incnum_equations]:= x[TVABSSinSource(Cir[i]).getp];
        end;
        if (TVABSSinSource(Cir[i]).getn <> Datum) then
        begin
           MNA[TVABSSinSource(Cir[i]).getn]:= MNA[TVABSSinSource(Cir[i]).getn]+x[num_nodes+incnum_equations];
           MNA[num_nodes+incnum_equations]:= MNA[num_nodes+incnum_equations]-x[TVABSSinSource(Cir[i]).getn];
        end;
      MNA[num_nodes+incnum_equations]:= MNA[num_nodes+incnum_equations]-TVABSSinSource(Cir[i]).getV;
      end;
    if (TElement(Cir[i]) is TDCISource) then
      begin
        if (TDCISource(Cir[i]).getp <> Datum) then
          MNA[TDCISource(Cir[i]).getp]:= MNA[TDCISource(Cir[i]).getp]-TDCISource(Cir[i]).getI;
        if (TDCISource(Cir[i]).getn <> Datum) then
          MNA[TDCISource(Cir[i]).getn]:= MNA[TDCISource(Cir[i]).getn]+TDCISource(Cir[i]).getI;
      end;
    if (TElement(Cir[i]) is TResistor) then
      begin
        if (TResistor(Cir[i]).getp <> Datum) then
           MNA[TResistor(Cir[i]).getp]:= MNA[TResistor(Cir[i]).getp]+TResistor(Cir[i]).getI;
        if (TResistor(Cir[i]).getn <> Datum) then
           MNA[TResistor(Cir[i]).getn]:= MNA[TResistor(Cir[i]).getn]-TResistor(Cir[i]).getI;
      end;
     if (TElement(Cir[i]) is TChuaResistor) then
      begin
        if (TChuaResistor(Cir[i]).getp <> Datum) then
           MNA[TChuaResistor(Cir[i]).getp]:= MNA[TChuaResistor(Cir[i]).getp]+TChuaResistor(Cir[i]).getI;
        if (TChuaResistor(Cir[i]).getn <> Datum) then
           MNA[TChuaResistor(Cir[i]).getn]:= MNA[TChuaResistor(Cir[i]).getn]-TChuaResistor(Cir[i]).getI;
      end;
     if (TElement(Cir[i]) is TDiode) then
      begin
        if (TDiode(Cir[i]).getp <> Datum) then
           MNA[TDiode(Cir[i]).getp]:= MNA[TDiode(Cir[i]).getp]+TDiode(Cir[i]).getI;
        if (TDiode(Cir[i]).getn <> Datum) then
           MNA[TDiode(Cir[i]).getn]:= MNA[TDiode(Cir[i]).getn]-TDiode(Cir[i]).getI;
      end;
     if (TElement(Cir[i]) is TDiodeZener) then
      begin
        if (TDiodeZener(Cir[i]).getp <> Datum) then
           MNA[TDiodeZener(Cir[i]).getp]:= MNA[TDiodeZener(Cir[i]).getp]+TDiodeZener(Cir[i]).getI;
        if (TDiodeZener(Cir[i]).getn <> Datum) then
           MNA[TDiodeZener(Cir[i]).getn]:= MNA[TDiodeZener(Cir[i]).getn]-TDiodeZener(Cir[i]).getI;
      end;
     if (TElement(Cir[i]) is TDiodeModelica) then
      begin
        if (TDiodeModelica(Cir[i]).getp <> Datum) then
           MNA[TDiodeModelica(Cir[i]).getp]:= MNA[TDiodeModelica(Cir[i]).getp]+TDiodeModelica(Cir[i]).getI;
        if (TDiodeModelica(Cir[i]).getn <> Datum) then
           MNA[TDiodeModelica(Cir[i]).getn]:= MNA[TDiodeModelica(Cir[i]).getn]-TDiodeModelica(Cir[i]).getI;
      end;
     if (TElement(Cir[i]) is TCapacitor) then
      begin
        if (TCapacitor(Cir[i]).getp <> Datum) then
           MNA[TCapacitor(Cir[i]).getp]:= MNA[TCapacitor(Cir[i]).getp]+TCapacitor(Cir[i]).getI;
        if (TCapacitor(Cir[i]).getn <> Datum) then
           MNA[TCapacitor(Cir[i]).getn]:= MNA[TCapacitor(Cir[i]).getn]-TCapacitor(Cir[i]).getI;
      end;
     if (TElement(Cir[i]) is TInductor) then
      begin
        if (TInductor(Cir[i]).getp <> Datum) then
        inc(incnum_equations);
          begin
           MNA[TInductor(Cir[i]).getp]:= MNA[TInductor(Cir[i]).getp]+x[num_nodes+incnum_equations];
           MNA[num_nodes+incnum_equations]:= x[TVSinSource(Cir[i]).getp];
          end;
        if (TInductor(Cir[i]).getn <> Datum) then
          begin
           MNA[TInductor(Cir[i]).getn]:= MNA[TInductor(Cir[i]).getn]-x[num_nodes+incnum_equations];
           MNA[num_nodes+incnum_equations]:= MNA[num_nodes+incnum_equations]-x[TVSinSource(Cir[i]).getn];
          end;
      end;
     if (TElement(Cir[i]) is TBJTNPNF) then
      begin
        if (TBJTNPNF(Cir[i]).getb <> Datum) then
           MNA[TBJTNPNF(Cir[i]).getb]:= MNA[TBJTNPNF(Cir[i]).getb]+TBJTNPNF(Cir[i]).getBi;
        if (TBJTNPNF(Cir[i]).getc <> Datum) then
           MNA[TBJTNPNF(Cir[i]).getc]:= MNA[TBJTNPNF(Cir[i]).getc]+TBJTNPNF(Cir[i]).getCi;
        if (TBJTNPNF(Cir[i]).gete <> Datum) then
           MNA[TBJTNPNF(Cir[i]).gete]:= MNA[TBJTNPNF(Cir[i]).gete]+TBJTNPNF(Cir[i]).getEi;
      end;
     if (TElement(Cir[i]) is TBJTEMNPN) then
      begin
        if (TBJTEMNPN(Cir[i]).getb <> Datum) then
           MNA[TBJTEMNPN(Cir[i]).getb]:= MNA[TBJTEMNPN(Cir[i]).getb]+TBJTEMNPN(Cir[i]).getBi;
        if (TBJTEMNPN(Cir[i]).getc <> Datum) then
           MNA[TBJTEMNPN(Cir[i]).getc]:= MNA[TBJTEMNPN(Cir[i]).getc]+TBJTEMNPN(Cir[i]).getCi;
        if (TBJTEMNPN(Cir[i]).gete <> Datum) then
           MNA[TBJTEMNPN(Cir[i]).gete]:= MNA[TBJTEMNPN(Cir[i]).gete]-TBJTEMNPN(Cir[i]).getEi;
      end;
     if (TElement(Cir[i]) is TBJTPWLNPN) then
      begin
        if (TBJTPWLNPN(Cir[i]).getb <> Datum) then
           MNA[TBJTPWLNPN(Cir[i]).getb]:= MNA[TBJTPWLNPN(Cir[i]).getb]+TBJTPWLNPN(Cir[i]).getBi;
        if (TBJTEMNPN(Cir[i]).getc <> Datum) then
           MNA[TBJTPWLNPN(Cir[i]).getc]:= MNA[TBJTPWLNPN(Cir[i]).getc]+TBJTPWLNPN(Cir[i]).getCi;
        if (TBJTEMNPN(Cir[i]).gete <> Datum) then
           MNA[TBJTPWLNPN(Cir[i]).gete]:= MNA[TBJTPWLNPN(Cir[i]).gete]-TBJTPWLNPN(Cir[i]).getEi;
      end;
     if (TElement(Cir[i]) is TBJTPWLPNP) then
      begin
        if (TBJTPWLPNP(Cir[i]).getb <> Datum) then
           MNA[TBJTPWLPNP(Cir[i]).getb]:= MNA[TBJTPWLPNP(Cir[i]).getb]-TBJTPWLNPN(Cir[i]).getBi;
        if (TBJTEMNPN(Cir[i]).getc <> Datum) then
           MNA[TBJTPWLPNP(Cir[i]).getc]:= MNA[TBJTPWLPNP(Cir[i]).getc]-TBJTPWLNPN(Cir[i]).getCi;
        if (TBJTEMNPN(Cir[i]).gete <> Datum) then
           MNA[TBJTPWLPNP(Cir[i]).gete]:= MNA[TBJTPWLPNP(Cir[i]).gete]+TBJTPWLPNP(Cir[i]).getEi;
      end;
     if (TElement(Cir[i]) is TBJTEMPNP) then
      begin
        if (TBJTEMPNP(Cir[i]).getb <> Datum) then
           MNA[TBJTEMPNP(Cir[i]).getb]:= MNA[TBJTEMPNP(Cir[i]).getb]-TBJTEMPNP(Cir[i]).getBi;
        if (TBJTEMPNP(Cir[i]).getc <> Datum) then
           MNA[TBJTEMPNP(Cir[i]).getc]:= MNA[TBJTEMPNP(Cir[i]).getc]-TBJTEMPNP(Cir[i]).getCi;
        if (TBJTEMPNP(Cir[i]).gete <> Datum) then
           MNA[TBJTEMPNP(Cir[i]).gete]:= MNA[TBJTEMPNP(Cir[i]).gete]+TBJTEMPNP(Cir[i]).getEi;
      end;
     if (TElement(Cir[i]) is TBJTPNPF) then
      begin
        if (TBJTPNPF(Cir[i]).getb <> Datum) then
           MNA[TBJTPNPF(Cir[i]).getb]:= MNA[TBJTPNPF(Cir[i]).getb]-TBJTPNPF(Cir[i]).getBi;
        if (TBJTPNPF(Cir[i]).getc <> Datum) then
           MNA[TBJTPNPF(Cir[i]).getc]:= MNA[TBJTPNPF(Cir[i]).getc]-TBJTPNPF(Cir[i]).getCi;
        if (TBJTPNPF(Cir[i]).gete <> Datum) then
           MNA[TBJTPNPF(Cir[i]).gete]:= MNA[TBJTPNPF(Cir[i]).gete]-TBJTPNPF(Cir[i]).getEi;
      end;
     if (TElement(Cir[i]) is TBJTNPN) then
      begin
        if (TBJTNPN(Cir[i]).getb <> Datum) then
           MNA[TBJTNPN(Cir[i]).getb]:= MNA[TBJTNPN(Cir[i]).getb]+TBJTNPN(Cir[i]).getBi;
        if (TBJTNPN(Cir[i]).getc <> Datum) then
           MNA[TBJTNPN(Cir[i]).getc]:= MNA[TBJTNPN(Cir[i]).getc]+TBJTNPN(Cir[i]).getCi;
        if (TBJTNPN(Cir[i]).gete <> Datum) then
           MNA[TBJTNPN(Cir[i]).gete]:= MNA[TBJTNPN(Cir[i]).gete]+TBJTNPN(Cir[i]).getEi;
      end;
     if (TElement(Cir[i]) is TBJTPNP) then
      begin
        if (TBJTPNP(Cir[i]).getb <> Datum) then
           MNA[TBJTPNP(Cir[i]).getb]:= MNA[TBJTPNP(Cir[i]).getb]+TBJTPNP(Cir[i]).getBi;
        if (TBJTPNP(Cir[i]).getc <> Datum) then
           MNA[TBJTPNP(Cir[i]).getc]:= MNA[TBJTPNP(Cir[i]).getc]+TBJTPNP(Cir[i]).getCi;
        if (TBJTPNP(Cir[i]).gete <> Datum) then
           MNA[TBJTPNP(Cir[i]).gete]:= MNA[TBJTPNP(Cir[i]).gete]+TBJTPNP(Cir[i]).getEi;
      end;
     if (TElement(Cir[i]) is TOPAMPIDEAL) then
      begin
      inc(incnum_equations);
        if (TOPAMPIDEAL(Cir[i]).getinp <> Datum) then
          MNA[num_nodes+incnum_equations]:= x[TOPAMPIDEAL(Cir[i]).getinp];
        if (TOPAMPIDEAL(Cir[i]).getinn <> Datum) then
          MNA[num_nodes+incnum_equations]:= MNA[num_nodes+incnum_equations]-x[TOPAMPIDEAL(Cir[i]).getinn];
        MNA[TOPAMPIDEAL(Cir[i]).getout]:= MNA[TOPAMPIDEAL(Cir[i]).getout]-x[num_nodes+incnum_equations];
      end;
     if (TElement(Cir[i]) is TOPAMP) then
      begin
      inc(incnum_equations);
      MNA[num_nodes+incnum_equations]:= x[TOPAMP(Cir[i]).getout]-TOPAMP(Cir[i]).getV;
      MNA[TOPAMP(Cir[i]).getout]:= MNA[TOPAMP(Cir[i]).getout]-x[num_nodes+incnum_equations];
      end;
     if (TElement(Cir[i]) is TOPAMP741) then
      begin
      //inc(incnum_equations,21);
      end;
     if (TElement(Cir[i]) is TVCT) then
      begin
        if (TVCT(Cir[i]).getoutp<>Datum) then
          MNA[TVCT(Cir[i]).getoutp]:= MNA[TVCT(Cir[i]).getoutp]+TVCT(Cir[i]).getI;
        if (TVCT(Cir[i]).getoutn<>Datum) then
          MNA[TVCT(Cir[i]).getoutn]:= MNA[TVCT(Cir[i]).getoutn]-TVCT(Cir[i]).getI;
      end;
     if (TElement(Cir[i]) is TVVT) then
      begin
      inc(incnum_equations);
        if (TVVT(Cir[i]).getoutn<>Datum) then
        begin
           MNA[TVVT(Cir[i]).getoutn]:= MNA[TVVT(Cir[i]).getoutn]-x[num_nodes+incnum_equations];
           MNA[num_nodes+incnum_equations]:= -x[TVVT(Cir[i]).getoutn];
        end;
        if (TVVT(Cir[i]).getoutp<>Datum) then
        begin
           MNA[TVVT(Cir[i]).getoutp]:= MNA[TVVT(Cir[i]).getoutp]-x[num_nodes+incnum_equations];
           MNA[num_nodes+incnum_equations]:= MNA[num_nodes+incnum_equations]+x[TVVT(Cir[i]).getoutp];
        end;
      MNA[num_nodes+incnum_equations]:= MNA[num_nodes+incnum_equations]-TVVT(Cir[i]).getV;
      end;
  inc(i);
  end; // end while
end;

function TCircuit.getSize: integer;
var i: integer;
begin
  num_nodess:= num_nodes+1;
  i:= 0;
  if firstpass then
    while (i < Cir.Count) do
    inc(i);
  firstpass:= False;
  result:= num_equations;
end;

function TCircuit.getNum_Nodes: integer;
begin
  getsize;
  result:= num_nodess-num_opamps;
end;

procedure TCircuit.ShowSolution;
var i: integer;
begin
   for i:=0 to (num_nodess+num_inductors+num_vsources-21*num_opamps-1) do
   begin
    //write('x[');write(i+1);write(']:= ');
    //writeln(result[i]:1:8);
   end;
end;

function TCircuit.getNonlinearity: Boolean;
begin
  result:= nonlinear;
end;

procedure TCircuit.setNonlinearity(no_linear: boolean);
begin
  nonlinear:= no_linear;
end;

procedure TCircuit.Jacobian(t: TSample; x, dxdt: TVec; Jac: TMtx);
var
   i:integer;
   temp, fun, funi: TVec;
begin
 temp:= TVec.Create;
 temp.Size(num_equations);
 fun:= TVec.Create;
 fun.Size(num_equations);
 funi:= TVec.Create;
 funi.Size(num_equations);
 DaeFunction(t,x,dxdt,fun);
 for i:=0 to num_equations-1 do
  begin
    temp.Assign(x);
    temp[i]:=temp[i]+1e-7;
    temp.Add(x);
    DaeFunction(t,temp,dxdt,funi); //F(t,y+diei,alfa*(y+diei)+beta)
    funi.Sub(fun);                 //fun1:= F(t,y,alfa*y+beta)
    funi.Scale(1/1e-7);
    Jac.SetCol(funi,i);
  end;
end;

function TCircuit.DCAnalysis: TVec;
var
  i: integer;
  t: double;
  x, dxdt, B: TVec;
begin
  t:= 0;
  x:= TVec.Create;
  x.Size(num_equations);
  dxdt:= TVec.Create;
  dxdt.Size(num_equations);
  B:= TVec.Create;
  B.Size(num_equations);
  Jac:= TMtx.Create;
  Jac.Size(num_equations,num_equations);
  Jacobian(t, x, dxdt, Jac);
  DaeFunction(t,x,dxdt,B);
  B.Scale(-1);
  result:= TVec(x);
  Jac.AdaptiveSGMRES(B, result);
end;

end.
