unit CircuitProblem;

interface

uses MNA, ComponentsMNA;

type
   TCircuitProblem = Class(TMNA)
   public
   VDC1, VDC2: TDCVSource;
   Norton1: TNorton;
   IDC1: TDCISource;
   R1, R2, R3, R4: TResistor;
   constructor Create; 
   end;

   TCircuitProblem2 = Class(TMNA)
   public
   IDC1, IDC2, IDC3: TDCISource;
   R1, R2, R3, R4, R5: TResistor;
   constructor Create; 
   end;

   TLinearTransistor = Class(TMNA)
   public
   RB, RC, RE: TResistor;
   VCC: TDCVSource;
   Q1: TLinearBJTNPN;
   constructor Create;
   end;

   TLinearAmplifier = Class(TMNA)
   public
   RB1, RB2, RE, RTH1, RTH2: TResistor;
   VCC, VEE, Vi: TDCVSource;
   Q1: TLinearBJTNPN;
   Q2: TLinearBJTPNP;
   constructor Create;
   end;

implementation

{ TCircuitProblem }

constructor TCircuitProblem.Create;
begin
  inherited Create;
  VDC1:= TDCVSource.Create(1,0,5.0);
  VDC2:= TDCVSource.Create(3,0,10.0);
  IDC1:= TDCISource.Create(2,0,3.0);
  Norton1:= TNorton.Create(2,0,4,0);
  R1:= TResistor.Create(1,2,4.0);
  R2:= TResistor.Create(2,3,5.0);
  R3:= TResistor.Create(2,0,6.0);
  R4:= TResistor.Create(4,0,7.0);
  Add(VDC1);
  Add(VDC2);
  Add(IDC1);
  Add(Norton1);
  Add(R1);
  Add(R2);
  Add(R3);
  Add(R4);
end;

{ TCircuitProblem2 }

constructor TCircuitProblem2.Create;
begin
  inherited Create;
  IDC1:= TDCISource.Create(1,0,20.0);
  IDC2:= TDCISource.Create(3,0,25.0);
  IDC3:= TDCISource.Create(2,0,10.0);
  R1:= TResistor.Create(1,2,4.0);
  R2:= TResistor.Create(2,3,4.0);
  R3:= TResistor.Create(2,0,1.0);
  R4:= TResistor.Create(1,0,1.0);
  R5:= TResistor.Create(3,0,1.0);
  Add(IDC1);
  Add(IDC2);
  Add(IDC3);
  Add(R1);
  Add(R2);
  Add(R3);
end;

{ TLinearTransistor }

constructor TLinearTransistor.Create;
begin
  inherited Create;
  VCC:= TDCVSource.Create(1,0,20.0);
  RB:= TResistor.Create(1,2,250e3);
  RC:= TResistor.Create(1,3,1000.0);
  RE:= TResistor.Create(4,0,10.0);
  Q1:= TLinearBJTNPN.Create(2,3,4);
  Add(VCC);
  Add(RB);
  Add(RC);
  Add(RE);
  Add(Q1);
end;

{ TLinearAmplifier }

constructor TLinearAmplifier.Create;
begin
  inherited Create;
  VCC:= TDCVSource.Create(1,0,12.0);
  VEE:= TDCVSource.Create(5,0,-12.0);
  Vi:=  TDCVSource.Create(6,0,4.5);
  RB1:= TResistor.Create(1,2,37000.0);
  RB2:= TResistor.Create(3,5,37000.0);
  RE:= TResistor.Create(4,0,100.0);
  RTH1:= TResistor.Create(2,6,2350.0);
  RTH2:= TResistor.Create(6,3,2350.0);
  Q1:= TLinearBJTNPN.Create(2,1,4);
  Q2:= TLinearBJTPNP.Create(3,5,4);
  Add(VCC);
  Add(VEE);
  Add(Vi);
  Add(RB1);
  Add(RB2);
  Add(RTH1);
  Add(RTH2);
  Add(RE);
  Add(Q1);
  Add(Q2);
end;

end.
