unit CuBlas_API;

interface

uses UCudaRT_V2;

(*
 * Copyright 1993-2011 NVIDIA Corporation.  All rights reserved.
 *
 * NOTICE TO LICENSEE:
 *
 * This source code and/or documentation ("Licensed Deliverables") are
 * subject to NVIDIA intellectual property rights under U.S. and
 * international Copyright laws.
 *
 * These Licensed Deliverables contained herein is PROPRIETARY and
 * CONFIDENTIAL to NVIDIA and is being provided under the terms and
 * conditions of a form of NVIDIA software license agreement by and
 * between NVIDIA and Licensee ("License Agreement") or electronically
 * accepted by Licensee.  Notwithstanding any terms or conditions to
 * the contrary in the License Agreement, reproduction or disclosure
 * of the Licensed Deliverables to any third party without the express
 * written consent of NVIDIA is prohibited.
 *
 * NOTWITHSTANDING ANY TERMS OR CONDITIONS TO THE CONTRARY IN THE
 * LICENSE AGREEMENT, NVIDIA MAKES NO REPRESENTATION ABOUT THE
 * SUITABILITY OF THESE LICENSED DELIVERABLES FOR ANY PURPOSE.  IT IS
 * PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTY OF ANY KIND.
 * NVIDIA DISCLAIMS ALL WARRANTIES WITH REGARD TO THESE LICENSED
 * DELIVERABLES, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY,
 * NONINFRINGEMENT, AND FITNESS FOR A PARTICULAR PURPOSE.
 * NOTWITHSTANDING ANY TERMS OR CONDITIONS TO THE CONTRARY IN THE
 * LICENSE AGREEMENT, IN NO EVENT SHALL NVIDIA BE LIABLE FOR ANY
 * SPECIAL, INDIRECT, INCIDENTAL, OR CONSEQUENTIAL DAMAGES, OR ANY
 * DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS
 * ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE
 * OF THESE LICENSED DELIVERABLES.
 *
 * U.S. Government End Users.  These Licensed Deliverables are a
 * "commercial item" as that term is defined at 48 C.F.R. 2.101 (OCT
 * 1995), consisting of "commercial computer software" and "commercial
 * computer software documentation" as such terms are used in 48
 * C.F.R. 12.212 (SEPT 1995) and is provided to the U.S. Government
 * only as a commercial end item.  Consistent with 48 C.F.R.12.212 and
 * 48 C.F.R. 227.7202-1 through 227.7202-4 (JUNE 1995), all
 * U.S. Government End Users acquire the Licensed Deliverables with
 * only those rights set forth herein.
 *
 * Any use of the Licensed Deliverables in individual and commercial
 * software must include, in the user documentation and internal
 * comments to the code, the above Disclaimer and U.S. Government End
 * Users Notice.
 *)

(*
 * This is the public header file for the CUBLAS library, defining the API
 *
 * CUBLAS is an implementation of BLAS (Basic Linear Algebra Subroutines)
 * on top of the CUDA runtime.
 *)


// #define CUBLASAPI __stdcall

const
  cublas_v2dll = 'cublas32_40_17.dll';

(* CUBLAS status type returns *)
type
  (* CUBLAS data types *)

  cublasDouble = Double;

  TSingleVector = array[0..0] of Single;
  PSingleVector = ^TSingleVector;
  TDoubleVector = array[0..0] of Double;
  PDoubleVector = ^TDoubleVector;
  TIntegerVector = array[0..0] of Integer;
  PIntegerVector = ^TIntegerVector;


  cublasStatus_t = (
	 CUBLAS_STATUS_SUCCESS	= 0,
	 CUBLAS_STATUS_NOT_INITIALIZED	= 1,
	 CUBLAS_STATUS_ALLOC_FAILED	= 3,
	 CUBLAS_STATUS_INVALID_VALUE	= 7,
	 CUBLAS_STATUS_ARCH_MISMATCH	= 8,
	 CUBLAS_STATUS_MAPPING_ERROR	= 11,
	 CUBLAS_STATUS_EXECUTION_FAILED	= 13,
	 CUBLAS_STATUS_INTERNAL_ERROR	= 14
	 );


  cublasFillMode_t = (
	 CUBLAS_FILL_MODE_LOWER	= 0,
	 CUBLAS_FILL_MODE_UPPER	= 1
	 );

  cublasDiagType_t = (
	 CUBLAS_DIAG_NON_UNIT	= 0,
	 CUBLAS_DIAG_UNIT	= 1
	 );

  cublasSideMode_t = (
	 CUBLAS_SIDE_LEFT	= 0,
	 CUBLAS_SIDE_RIGHT	= 1
	 );


  cublasOperation_t = (
	 CUBLAS_OP_N	= 0,
	 CUBLAS_OP_T	= 1,
	 CUBLAS_OP_C	= 2
	 );


  cublasPointerMode_t = (
	 CUBLAS_POINTER_MODE_HOST	= 0,
	 CUBLAS_POINTER_MODE_DEVICE	= 1
	 );



(* Opaque structure holding CUBLAS library context *)

  cublasContext = record
  end;
  cublasHandle_t	= ^cublasContext;

  function cublasCreate_v2(var handle: cublasHandle_t): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasCreate_v2';
  function cublasDestroy_v2(handle: cublasHandle_t): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasDestroy_v2';
  function cublasGetVersion_v2(handle: cublasHandle_t; var version: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasGetVersion_v2';
  function cublasSetStream_v2(handle: cublasHandle_t; streamId: cudaStream_t): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasSetStream_v2';
  function cublasGetStream_v2(handle: cublasHandle_t; var streamId: cudaStream_t): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasGetStream_v2';

  function cublasGetPointerMode_v2(handle: cublasHandle_t; var mode: cublasPointerMode_t): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasGetPointerMode_v2';
  function cublasSetPointerMode_v2(handle: cublasHandle_t; mode: cublasPointerMode_t): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasSetPointerMode_v2';


(* 
 * cublasStatus_t 
 * cublasSetVector (int n, int elemSize, const void *x, int incx, 
 *                  void *y, int incy) 
 *
 * copies n elements from a vector x in CPU memory space to a vector y 
 * in GPU memory space. Elements in both vectors are assumed to have a 
 * size of elemSize bytes. Storage spacing between consecutive elements
 * is incx for the source vector x and incy for the destination vector
 * y. In general, y points to an object, or part of an object, allocated
 * via cublasAlloc(). Column major format for two-dimensional matrices
 * is assumed throughout CUBLAS. Therefore, if the increment for a vector 
 * is equal to 1, this access a column vector while using an increment 
 * equal to the leading dimension of the respective matrix accesses a 
 * row vector.
 *
 * Return Values
 * -------------
 * CUBLAS_STATUS_NOT_INITIALIZED  if CUBLAS library not been initialized
 * CUBLAS_STATUS_INVALID_VALUE    if incx, incy, or elemSize <= 0
 * CUBLAS_STATUS_MAPPING_ERROR    if an error occurred accessing GPU memory   
 * CUBLAS_STATUS_SUCCESS          if the operation completed successfully
 *)
 function cublasSetVector(n: integer; elemSize: integer; x: Pointer;
                         incx: integer; y: Pointer; incy: integer): cublasStatus_t;
                         stdcall; external cublas_v2dll name 'cublasSetVector';

(* 
 * cublasStatus_t 
 * cublasGetVector (int n, int elemSize, const void *x, int incx, 
 *                  void *y, int incy)
 * 
 * copies n elements from a vector x in GPU memory space to a vector y 
 * in CPU memory space. Elements in both vectors are assumed to have a 
 * size of elemSize bytes. Storage spacing between consecutive elements
 * is incx for the source vector x and incy for the destination vector
 * y. In general, x points to an object, or part of an object, allocated
 * via cublasAlloc(). Column major format for two-dimensional matrices
 * is assumed throughout CUBLAS. Therefore, if the increment for a vector 
 * is equal to 1, this access a column vector while using an increment 
 * equal to the leading dimension of the respective matrix accesses a 
 * row vector.
 *
 * Return Values
 * -------------
 * CUBLAS_STATUS_NOT_INITIALIZED  if CUBLAS library not been initialized
 * CUBLAS_STATUS_INVALID_VALUE    if incx, incy, or elemSize <= 0
 * CUBLAS_STATUS_MAPPING_ERROR    if an error occurred accessing GPU memory   
 * CUBLAS_STATUS_SUCCESS          if the operation completed successfully
 *)
  function cublasGetVector(n: integer; elemSize: integer; x: Pointer;
                          incx: Integer; y: Pointer; incy: integer): cublasStatus_t;
                          stdcall; external cublas_v2dll name 'cublasGetVector';

(*
 * cublasStatus_t 
 * cublasSetMatrix (int rows, int cols, int elemSize, const void *A, 
 *                  int lda, void *B, int ldb)
 *
 * copies a tile of rows x cols elements from a matrix A in CPU memory
 * space to a matrix B in GPU memory space. Each element requires storage
 * of elemSize bytes. Both matrices are assumed to be stored in column 
 * major format, with the leading dimension (i.e. number of rows) of 
 * source matrix A provided in lda, and the leading dimension of matrix B
 * provided in ldb. In general, B points to an object, or part of an 
 * object, that was allocated via cublasAlloc().
 *
 * Return Values 
 * -------------
 * CUBLAS_STATUS_NOT_INITIALIZED  if CUBLAS library has not been initialized
 * CUBLAS_STATUS_INVALID_VALUE    if rows or cols < 0, or elemSize, lda, or 
 *                                ldb <= 0
 * CUBLAS_STATUS_MAPPING_ERROR    if error occurred accessing GPU memory
 * CUBLAS_STATUS_SUCCESS          if the operation completed successfully
 *)
 function cublasSetMatrix(rows: integer; cols: integer; elemSize: integer;
                          A: Pointer; lda: integer; B: Pointer; ldb: integer): cublasStatus_t;
                          stdcall; external cublas_v2dll name 'cublasSetMatrix';

(*
 * cublasStatus_t 
 * cublasGetMatrix (int rows, int cols, int elemSize, const void *A, 
 *                  int lda, void *B, int ldb)
 *
 * copies a tile of rows x cols elements from a matrix A in GPU memory
 * space to a matrix B in CPU memory space. Each element requires storage
 * of elemSize bytes. Both matrices are assumed to be stored in column 
 * major format, with the leading dimension (i.e. number of rows) of 
 * source matrix A provided in lda, and the leading dimension of matrix B
 * provided in ldb. In general, A points to an object, or part of an 
 * object, that was allocated via cublasAlloc().
 *
 * Return Values 
 * -------------
 * CUBLAS_STATUS_NOT_INITIALIZED  if CUBLAS library has not been initialized
 * CUBLAS_STATUS_INVALID_VALUE    if rows, cols, eleSize, lda, or ldb <= 0
 * CUBLAS_STATUS_MAPPING_ERROR    if error occurred accessing GPU memory
 * CUBLAS_STATUS_SUCCESS          if the operation completed successfully
 *)
 function cublasGetMatrix (rows: integer; cols: integer; elemSize: integer;
                          A: Pointer; lda: integer; B: Pointer; ldb: integer): cublasStatus_t;
                          stdcall; external cublas_v2dll name 'cublasGetMatrix';

(* 
 * cublasStatus 
 * cublasSetVectorAsync ( int n, int elemSize, const void *x, int incx, 
 *                       void *y, int incy, cudaStream_t stream );
 *
 * cublasSetVectorAsync has the same functionnality as cublasSetVector
 * but the transfer is done asynchronously within the CUDA stream passed
 * in parameter.
 *
 * Return Values
 * -------------
 * CUBLAS_STATUS_NOT_INITIALIZED  if CUBLAS library not been initialized
 * CUBLAS_STATUS_INVALID_VALUE    if incx, incy, or elemSize <= 0
 * CUBLAS_STATUS_MAPPING_ERROR    if an error occurred accessing GPU memory   
 * CUBLAS_STATUS_SUCCESS          if the operation completed successfully
 *)
  function cublasSetVectorAsync(n: Integer; elemSize: Integer; hostPtr: Pointer;
	 incx: Integer; devicePtr: Pointer; incy: Integer; stream: cudaStream_t): cublasStatus_t;
                           stdcall; external cublas_v2dll name 'cublasSetVectorAsync';
(* 
 * cublasStatus 
 * cublasGetVectorAsync( int n, int elemSize, const void *x, int incx, 
 *                       void *y, int incy, cudaStream_t stream)
 * 
 * cublasGetVectorAsync has the same functionnality as cublasGetVector
 * but the transfer is done asynchronously within the CUDA stream passed
 * in parameter.
 *
 * Return Values
 * -------------
 * CUBLAS_STATUS_NOT_INITIALIZED  if CUBLAS library not been initialized
 * CUBLAS_STATUS_INVALID_VALUE    if incx, incy, or elemSize <= 0
 * CUBLAS_STATUS_MAPPING_ERROR    if an error occurred accessing GPU memory   
 * CUBLAS_STATUS_SUCCESS          if the operation completed successfully
 *)
  function cublasGetVectorAsync(n: Integer; elemSize: Integer; devicePtr: Pointer;
	 incx: Integer; hostPtr: Pointer; incy: Integer; stream: cudaStream_t): cublasStatus_t;
                            stdcall; external cublas_v2dll name 'cublasGetVectorAsync';

(*
 * cublasStatus_t 
 * cublasSetMatrixAsync (int rows, int cols, int elemSize, const void *A, 
 *                       int lda, void *B, int ldb, cudaStream_t stream)
 *
 * cublasSetMatrixAsync has the same functionnality as cublasSetMatrix
 * but the transfer is done asynchronously within the CUDA stream passed
 * in parameter.
 *
 * Return Values 
 * -------------
 * CUBLAS_STATUS_NOT_INITIALIZED  if CUBLAS library has not been initialized
 * CUBLAS_STATUS_INVALID_VALUE    if rows or cols < 0, or elemSize, lda, or 
 *                                ldb <= 0
 * CUBLAS_STATUS_MAPPING_ERROR    if error occurred accessing GPU memory
 * CUBLAS_STATUS_SUCCESS          if the operation completed successfully
 *)
  function cublasSetMatrixAsync(rows: Integer; cols: Integer; elemSize: Integer;
	 A: Pointer; lda: Integer; B: Pointer; ldb: Integer; stream: cudaStream_t): cublasStatus_t;
                                stdcall; external cublas_v2dll name 'cublasSetMatrixAsync';

(*
 * cublasStatus_t 
 * cublasGetMatrixAsync (int rows, int cols, int elemSize, const void *A, 
 *                       int lda, void *B, int ldb, cudaStream_t stream)
 *
 * cublasGetMatrixAsync has the same functionnality as cublasGetMatrix
 * but the transfer is done asynchronously within the CUDA stream passed
 * in parameter.
 *
 * Return Values 
 * -------------
 * CUBLAS_STATUS_NOT_INITIALIZED  if CUBLAS library has not been initialized
 * CUBLAS_STATUS_INVALID_VALUE    if rows, cols, eleSize, lda, or ldb <= 0
 * CUBLAS_STATUS_MAPPING_ERROR    if error occurred accessing GPU memory
 * CUBLAS_STATUS_SUCCESS          if the operation completed successfully
 *)
  function cublasGetMatrixAsync(rows: Integer; cols: Integer; elemSize: Integer;
	 A: Pointer; lda: Integer; B: Pointer; ldb: Integer; stream: cudaStream_t): cublasStatus_t;
                               stdcall; external cublas_v2dll name 'cublasGetMatrixAsync';


  procedure cublasXerbla(const srName: PChar; info: Integer); stdcall; external cublas_v2dll name 'cublasXerbla';

  
(* ---------------- CUBLAS BLAS1 functions ---------------- *)
  function cublasSnrm2_v2(handle: cublasHandle_t; n: Integer; x: PSingle;
	 incx: Integer; var result: Single): cublasStatus_t; stdcall;	external cublas_v2dll name 'cublasSnrm2_v2'; (* host or device pointer *)

  function cublasDnrm2_v2(handle: cublasHandle_t; n: Integer; x: PDouble;
	 incx: Integer; var result: double): cublasStatus_t; stdcall;	external cublas_v2dll name 'cublasDnrm2_v2';(* host or device pointer *)

  //function cublasScnrm2_v2(handle: cublasHandle_t; n: Integer; x: p_cuComplex;
	// incx: Integer; var result: Single): cublasStatus_t; stdcall;	(* host or device pointer *)

  //function cublasDznrm2_v2(handle: cublasHandle_t; n: Integer; const x: p_cuDoubleComplex;
	// incx: Integer; var result: double): cublasStatus_t; stdcall;	(* host or device pointer *)

  function cublasSdot_v2(handle: cublasHandle_t; n: Integer; x: PSingle;
	 incx: Integer; const y: PSingle; incy: Integer; var result: Single): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasSdot_v2';	(* host or device pointer *)

  function cublasDdot_v2(handle: cublasHandle_t; n: Integer; const x: PDouble;
	 incx: Integer; const y: PDouble; incy: Integer; var result: double): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasDdot_v2';	(* host or device pointer *)

  //function cublasCdotu_v2(handle: cublasHandle_t; n: Integer; const x: p_cuComplex;
	// incx: Integer; const y: p_cuComplex; incy: Integer; var result: cuComplex): cublasStatus_t; stdcall;	(* host or device pointer *)

  //function cublasCdotc_v2(handle: cublasHandle_t; n: Integer; const x: p_cuComplex;
	// incx: Integer; const y: p_cuComplex; incy: Integer; var result: cuComplex): cublasStatus_t; stdcall;	(* host or device pointer *)

  //function cublasZdotu_v2(handle: cublasHandle_t; n: Integer; const x: p_cuDoubleComplex;
	// incx: Integer; const y: p_cuDoubleComplex; incy: Integer; var result: cuDoubleComplex): cublasStatus_t; stdcall;	(* host or device pointer *)

  //function cublasZdotc_v2(handle: cublasHandle_t; n: Integer; const x: p_cuDoubleComplex;
	// incx: Integer; const y: p_cuDoubleComplex; incy: Integer; var result: cuDoubleComplex): cublasStatus_t; stdcall;	(* host or device pointer *)

  function cublasSscal_v2(handle: cublasHandle_t; n: Integer; const alpha: PSingle; (* host or device pointer *)
	 var x: Single; incx: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasSscal_v2';

  function cublasDscal_v2(handle: cublasHandle_t; n: Integer; const alpha: PDouble; (* host or device pointer *)
	 var x: double; incx: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasDscal_v2';

  //function cublasCscal_v2(handle: cublasHandle_t; n: Integer; const alpha: p_cuComplex; (* host or device pointer *)
	// var x: cuComplex; incx: Integer): cublasStatus_t; stdcall;

  //function cublasCsscal_v2(handle: cublasHandle_t; n: Integer; const alpha: PSingle; (* host or device pointer *)
	// var x: cuComplex; incx: Integer): cublasStatus_t; stdcall;

  //function cublasZscal_v2(handle: cublasHandle_t; n: Integer; const alpha: p_cuDoubleComplex; (* host or device pointer *)
	// var x: cuDoubleComplex; incx: Integer): cublasStatus_t; stdcall;

  //function cublasZdscal_v2(handle: cublasHandle_t; n: Integer; const alpha: PDouble; (* host or device pointer *)
	// var x: cuDoubleComplex; incx: Integer): cublasStatus_t; stdcall;

  function cublasSaxpy_v2(handle: cublasHandle_t; n: Integer; const alpha: PSingle; (* host or device pointer *)
	 const x: PSingle; incx: Integer; var y: Single; incy: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasSaxpy_v2';

  function cublasDaxpy_v2(handle: cublasHandle_t; n: Integer; const alpha: PDouble; (* host or device pointer *)
	 const x: PDouble; incx: Integer; var y: double; incy: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasDaxpy_v2';

  //function cublasCaxpy_v2(handle: cublasHandle_t; n: Integer; const alpha: p_cuComplex; (* host or device pointer *)
	// const x: p_cuComplex; incx: Integer; var y: cuComplex; incy: Integer): cublasStatus_t; stdcall;

  //function cublasZaxpy_v2(handle: cublasHandle_t; n: Integer; const alpha: p_cuDoubleComplex; (* host or device pointer *)
	// const x: p_cuDoubleComplex; incx: Integer; var y: cuDoubleComplex; incy: Integer): cublasStatus_t; stdcall;

  function cublasScopy_v2(handle: cublasHandle_t; n: Integer; const x: PSingle;
	 incx: Integer; var y: Single; incy: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasScopy_v2';

  function cublasDcopy_v2(handle: cublasHandle_t; n: Integer; const x: PDouble;
	 incx: Integer; var y: double; incy: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasDcopy_v2';

  //function cublasCcopy_v2(handle: cublasHandle_t; n: Integer; const x: p_cuComplex;
	// incx: Integer; var y: cuComplex; incy: Integer): cublasStatus_t; stdcall;

  //function cublasZcopy_v2(handle: cublasHandle_t; n: Integer; const x: p_cuDoubleComplex;
	// incx: Integer; var y: cuDoubleComplex; incy: Integer): cublasStatus_t; stdcall;

  function cublasSswap_v2(handle: cublasHandle_t; n: Integer; var x: Single; incx: Integer;
	 var y: Single; incy: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasSswap_v2';

  function cublasDswap_v2(handle: cublasHandle_t; n: Integer; var x: double; incx: Integer;
	 var y: double; incy: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasDswap_v2';

  //function cublasCswap_v2(handle: cublasHandle_t; n: Integer; var x: cuComplex;
	// incx: Integer; var y: cuComplex; incy: Integer): cublasStatus_t; stdcall;

  //function cublasZswap_v2(handle: cublasHandle_t; n: Integer; var x: cuDoubleComplex;
	// incx: Integer; var y: cuDoubleComplex; incy: Integer): cublasStatus_t; stdcall;

  function cublasIsamax_v2(handle: cublasHandle_t; n: Integer; const x: PSingle;
	 incx: Integer; var result: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasIsamax_v2';	(* host or device pointer *)

  function cublasIdamax_v2(handle: cublasHandle_t; n: Integer; const x: PDouble;
	 incx: Integer; var result: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasIdamax_v2';	(* host or device pointer *)

  //function cublasIcamax_v2(handle: cublasHandle_t; n: Integer; const x: p_cuComplex;
	// incx: Integer; var result: Integer): cublasStatus_t; stdcall;	(* host or device pointer *)

  //function cublasIzamax_v2(handle: cublasHandle_t; n: Integer; const x: p_cuDoubleComplex;
	// incx: Integer; var result: Integer): cublasStatus_t; stdcall;	(* host or device pointer *)

  function cublasIsamin_v2(handle: cublasHandle_t; n: Integer; const x: PSingle;
	 incx: Integer; var result: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasIsamin_v2';	(* host or device pointer *)

  function cublasIdamin_v2(handle: cublasHandle_t; n: Integer; const x: PDouble;
	 incx: Integer; var result: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasIdamin_v2';	(* host or device pointer *)

  //function cublasIcamin_v2(handle: cublasHandle_t; n: Integer; const x: p_cuComplex;
	// incx: Integer; var result: Integer): cublasStatus_t; stdcall;	(* host or device pointer *)

  //function cublasIzamin_v2(handle: cublasHandle_t; n: Integer; const x: p_cuDoubleComplex;
	// incx: Integer; var result: Integer): cublasStatus_t; stdcall;	(* host or device pointer *)

  function cublasSasum_v2(handle: cublasHandle_t; n: Integer; const x: PSingle;
	 incx: Integer; var result: Single): cublasStatus_t; stdcall;	external cublas_v2dll name 'cublasSasum_v2';(* host or device pointer *)

  function cublasDasum_v2(handle: cublasHandle_t; n: Integer; const x: PDouble;
	 incx: Integer; var result: double): cublasStatus_t; stdcall;	external cublas_v2dll name 'cublasDasum_v2';(* host or device pointer *)

  //function cublasScasum_v2(handle: cublasHandle_t; n: Integer; const x: p_cuComplex;
	// incx: Integer; var result: Single): cublasStatus_t; stdcall;	(* host or device pointer *)

  //function cublasDzasum_v2(handle: cublasHandle_t; n: Integer; const x: p_cuDoubleComplex;
	// incx: Integer; var result: double): cublasStatus_t; stdcall;	(* host or device pointer *)

  function cublasSrot_v2(handle: cublasHandle_t; n: Integer; var x: Single; incx: Integer;
	 var y: Single; incy: Integer; const c: PSingle; (* host or device pointer *)
	 const s: PSingle): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasSrot_v2';	(* host or device pointer *)

  function cublasDrot_v2(handle: cublasHandle_t; n: Integer; var x: double; incx: Integer;
	 var y: double; incy: Integer; const c: PDouble; (* host or device pointer *)
	 const s: PDouble): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasDrot_v2';	(* host or device pointer *)

  //function cublasCrot_v2(handle: cublasHandle_t; n: Integer; var x: cuComplex;
	// incx: Integer; var y: cuComplex; incy: Integer; const c: PSingle; (* host or device pointer *)
	// const s: p_cuComplex): cublasStatus_t; stdcall;	(* host or device pointer *)

  //function cublasCsrot_v2(handle: cublasHandle_t; n: Integer; var x: cuComplex;
	// incx: Integer; var y: cuComplex; incy: Integer; const c: PSingle; (* host or device pointer *)
	// const s: PSingle): cublasStatus_t; stdcall;	(* host or device pointer *)

  //function cublasZrot_v2(handle: cublasHandle_t; n: Integer; var x: cuDoubleComplex;
	// incx: Integer; var y: cuDoubleComplex; incy: Integer; const c: PDouble; (* host or device pointer *)
	// const s: p_cuDoubleComplex): cublasStatus_t; stdcall;	(* host or device pointer *)

  //function cublasZdrot_v2(handle: cublasHandle_t; n: Integer; var x: cuDoubleComplex;
	// incx: Integer; var y: cuDoubleComplex; incy: Integer; const c: PDouble; (* host or device pointer *)
	// const s: PDouble): cublasStatus_t; stdcall;	(* host or device pointer *)

  function cublasSrotg_v2(handle: cublasHandle_t; var a: Single; (* host or device pointer *)
	 var b: Single; (* host or device pointer *)
	 var c: Single; (* host or device pointer *)
	 var s: Single): cublasStatus_t; stdcall;	external cublas_v2dll name 'cublasSrotg_v2';(* host or device pointer *)

  function cublasDrotg_v2(handle: cublasHandle_t; var a: double; (* host or device pointer *)
	 var b: double; (* host or device pointer *)
	 var c: double; (* host or device pointer *)
	 var s: double): cublasStatus_t; stdcall;	external cublas_v2dll name 'cublasDrotg_v2';(* host or device pointer *)

  //function cublasCrotg_v2(handle: cublasHandle_t; var a: cuComplex; (* host or device pointer *)
	// var b: cuComplex; (* host or device pointer *)
	// var c: Single; (* host or device pointer *)
	// var s: cuComplex): cublasStatus_t; stdcall;	(* host or device pointer *)

  //function cublasZrotg_v2(handle: cublasHandle_t; var a: cuDoubleComplex; (* host or device pointer *)
	// var b: cuDoubleComplex; (* host or device pointer *)
	// var c: double; (* host or device pointer *)
	// var s: cuDoubleComplex): cublasStatus_t; stdcall;	(* host or device pointer *)

  function cublasSrotm_v2(handle: cublasHandle_t; n: Integer; var x: Single; incx: Integer;
	 var y: Single; incy: Integer; const param: PSingle): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasSrotm_v2';	(* host or device pointer *)

  function cublasDrotm_v2(handle: cublasHandle_t; n: Integer; var x: double; incx: Integer;
	 var y: double; incy: Integer; const param: PDouble): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasDrotm_v2';	(* host or device pointer *)

  function cublasSrotmg_v2(handle: cublasHandle_t; var d1: Single; (* host or device pointer *)
	 var d2: Single; (* host or device pointer *)
	 var x1: Single; (* host or device pointer *)
	 const y1: PSingle; (* host or device pointer *)
	 var param: Single): cublasStatus_t; stdcall;	external cublas_v2dll name 'cublasSrotmg_v2';(* host or device pointer *)

  function cublasDrotmg_v2(handle: cublasHandle_t; var d1: double; (* host or device pointer *)
	 var d2: double; (* host or device pointer *)
	 var x1: double; (* host or device pointer *)
	 const y1: PDouble; (* host or device pointer *)
	 var param: double): cublasStatus_t; stdcall;	external cublas_v2dll name 'cublasDrotmg_v2';(* host or device pointer *)

(* --------------- CUBLAS BLAS2 functions  ---------------- *)

(* GEMV *)
  function cublasSgemv_v2(handle: cublasHandle_t; trans: cublasOperation_t; m: Integer;
	 n: Integer; const alpha: PSingle; (* host or device pointer *)
	 const A: PSingle; lda: Integer; const x: PSingle; incx: Integer; const beta: PSingle; (* host or device pointer *)
	 var y: Single; incy: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasSgemv_v2';

  function cublasDgemv_v2(handle: cublasHandle_t; trans: cublasOperation_t; m: Integer;
	 n: Integer; const alpha: PDouble; (* host or device pointer *)
	 const A: PDouble; lda: Integer; const x: PDouble; incx: Integer; const beta: PDouble; (* host or device pointer *)
	 var y: double; incy: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasDgemv_v2';

  //function cublasCgemv_v2(handle: cublasHandle_t; trans: cublasOperation_t; m: Integer;
	// n: Integer; const alpha: p_cuComplex; (* host or device pointer *)
	// const A: p_cuComplex; lda: Integer; const x: p_cuComplex; incx: Integer; const beta: p_cuComplex; (* host or device pointer *)
	// var y: cuComplex; incy: Integer): cublasStatus_t; stdcall;

  //function cublasZgemv_v2(handle: cublasHandle_t; trans: cublasOperation_t; m: Integer;
	// n: Integer; const alpha: p_cuDoubleComplex; (* host or device pointer *)
	// const A: p_cuDoubleComplex; lda: Integer; const x: p_cuDoubleComplex; incx: Integer;
	// const beta: p_cuDoubleComplex; (* host or device pointer *)
	// var y: cuDoubleComplex; incy: Integer): cublasStatus_t; stdcall;

(* GBMV *)
  function cublasSgbmv_v2(handle: cublasHandle_t; trans: cublasOperation_t; m: Integer;
	 n: Integer; kl: Integer; ku: Integer; const alpha: PSingle; (* host or device pointer *)
	 const A: PSingle; lda: Integer; const x: PSingle; incx: Integer; const beta: PSingle; (* host or device pointer *)
	 var y: Single; incy: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasSgbmv_v2';

  function cublasDgbmv_v2(handle: cublasHandle_t; trans: cublasOperation_t; m: Integer;
	 n: Integer; kl: Integer; ku: Integer; const alpha: PDouble; (* host or device pointer *)
	 const A: PDouble; lda: Integer; const x: PDouble; incx: Integer; const beta: PDouble; (* host or device pointer *)
	 var y: double; incy: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasDgbmv_v2';

  //function cublasCgbmv_v2(handle: cublasHandle_t; trans: cublasOperation_t; m: Integer;
	// n: Integer; kl: Integer; ku: Integer; const alpha: p_cuComplex; (* host or device pointer *)
	// const A: p_cuComplex; lda: Integer; const x: p_cuComplex; incx: Integer; const beta: p_cuComplex; (* host or device pointer *)
	// var y: cuComplex; incy: Integer): cublasStatus_t; stdcall;

  //function cublasZgbmv_v2(handle: cublasHandle_t; trans: cublasOperation_t; m: Integer;
	// n: Integer; kl: Integer; ku: Integer; const alpha: p_cuDoubleComplex; (* host or device pointer *)
	// const A: p_cuDoubleComplex; lda: Integer; const x: p_cuDoubleComplex; incx: Integer;
	// const beta: p_cuDoubleComplex; (* host or device pointer *)
	// var y: cuDoubleComplex; incy: Integer): cublasStatus_t; stdcall;

(* TRMV *)
  function cublasStrmv_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; trans: cublasOperation_t;
	 diag: cublasDiagType_t; n: Integer; const A: PSingle; lda: Integer; var x: Single; 
	 incx: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasStrmv_v2';

  function cublasDtrmv_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; trans: cublasOperation_t;
	 diag: cublasDiagType_t; n: Integer; const A: PDouble; lda: Integer; var x: double; 
	 incx: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasDtrmv_v2';

  {function cublasCtrmv_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; trans: cublasOperation_t;
	 diag: cublasDiagType_t; n: Integer; const A: p_cuComplex; lda: Integer; var x: cuComplex; 
	 incx: Integer): cublasStatus_t; stdcall;

  function cublasZtrmv_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; trans: cublasOperation_t; 
	 diag: cublasDiagType_t; n: Integer; const A: p_cuDoubleComplex; lda: Integer;
	 var x: cuDoubleComplex; incx: Integer): cublasStatus_t; stdcall;   }

(* TBMV *)
  function cublasStbmv_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; trans: cublasOperation_t;
	 diag: cublasDiagType_t; n: Integer; k: Integer; const A: PSingle; lda: Integer; 
	 var x: Single; incx: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasStbmv_v2';

  function cublasDtbmv_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; trans: cublasOperation_t;
	 diag: cublasDiagType_t; n: Integer; k: Integer; const A: PDouble; lda: Integer; 
	 var x: double; incx: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasDtbmv_v2';

  {function cublasCtbmv_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; trans: cublasOperation_t;
	 diag: cublasDiagType_t; n: Integer; k: Integer; const A: p_cuComplex; lda: Integer; 
	 var x: cuComplex; incx: Integer): cublasStatus_t; stdcall;

  function cublasZtbmv_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; trans: cublasOperation_t; 
	 diag: cublasDiagType_t; n: Integer; k: Integer; const A: p_cuDoubleComplex; lda: Integer; 
	 var x: cuDoubleComplex; incx: Integer): cublasStatus_t; stdcall;   }


(* TPMV *)
  function cublasStpmv_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; trans: cublasOperation_t;
	 diag: cublasDiagType_t; n: Integer; const AP: PSingle; var x: Single; incx: Integer): cublasStatus_t;
   stdcall; external cublas_v2dll name 'cublasStpmv_v2';


  function cublasDtpmv_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; trans: cublasOperation_t;
	 diag: cublasDiagType_t; n: Integer; const AP: PDouble; var x: double; incx: Integer): cublasStatus_t;
   stdcall; external cublas_v2dll name 'cublasDtpmv_v2';

  {function cublasCtpmv_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; trans: cublasOperation_t;
	 diag: cublasDiagType_t; n: Integer; const AP: p_cuComplex; var x: cuComplex; 
	 incx: Integer): cublasStatus_t; stdcall;

  function cublasZtpmv_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; trans: cublasOperation_t; 
	 diag: cublasDiagType_t; n: Integer; const AP: p_cuDoubleComplex; var x: cuDoubleComplex; 
	 incx: Integer): cublasStatus_t; stdcall;   }

(* TRSV *)
  function cublasStrsv_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; trans: cublasOperation_t;
	 diag: cublasDiagType_t; n: Integer; const A: PSingle; lda: Integer; var x: Single; 
	 incx: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasStrsv_v2';

  function cublasDtrsv_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; trans: cublasOperation_t;
	 diag: cublasDiagType_t; n: Integer; const A: PDouble; lda: Integer; var x: double; 
	 incx: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasDtrsv_v2';

  {function cublasCtrsv_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; trans: cublasOperation_t;
	 diag: cublasDiagType_t; n: Integer; const A: p_cuComplex; lda: Integer; var x: cuComplex; 
	 incx: Integer): cublasStatus_t; stdcall;

  function cublasZtrsv_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; trans: cublasOperation_t; 
	 diag: cublasDiagType_t; n: Integer; const A: p_cuDoubleComplex; lda: Integer; 
	 var x: cuDoubleComplex; incx: Integer): cublasStatus_t; stdcall;     }

(* TPSV *)
  function cublasStpsv_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; trans: cublasOperation_t;
	 diag: cublasDiagType_t; n: Integer; const AP: PSingle; var x: Single; incx: Integer): cublasStatus_t;
   stdcall; external cublas_v2dll name 'cublasStpsv_v2';

  function cublasDtpsv_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; trans: cublasOperation_t;
	 diag: cublasDiagType_t; n: Integer; const AP: PDouble; var x: double; incx: Integer): cublasStatus_t;
   stdcall; external cublas_v2dll name 'cublasDtpsv_v2';

  {function cublasCtpsv_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; trans: cublasOperation_t;
	 diag: cublasDiagType_t; n: Integer; const AP: p_cuComplex; var x: cuComplex; 
	 incx: Integer): cublasStatus_t; stdcall;

  function cublasZtpsv_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; trans: cublasOperation_t; 
	 diag: cublasDiagType_t; n: Integer; const AP: p_cuDoubleComplex; var x: cuDoubleComplex; 
	 incx: Integer): cublasStatus_t; stdcall;            }
(* TBSV *)
  function cublasStbsv_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; trans: cublasOperation_t;
	 diag: cublasDiagType_t; n: Integer; k: Integer; const A: PSingle; lda: Integer; 
	 var x: Single; incx: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasStbsv_v2';

  function cublasDtbsv_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; trans: cublasOperation_t;
	 diag: cublasDiagType_t; n: Integer; k: Integer; const A: PDouble; lda: Integer; 
	 var x: double; incx: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasDtbsv_v2';

  {function cublasCtbsv_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; trans: cublasOperation_t;
	 diag: cublasDiagType_t; n: Integer; k: Integer; const A: p_cuComplex; lda: Integer; 
	 var x: cuComplex; incx: Integer): cublasStatus_t; stdcall;

  function cublasZtbsv_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; trans: cublasOperation_t; 
	 diag: cublasDiagType_t; n: Integer; k: Integer; const A: p_cuDoubleComplex; lda: Integer; 
	 var x: cuDoubleComplex; incx: Integer): cublasStatus_t; stdcall;    }

(* SYMV/HEMV *)
  function cublasSsymv_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; n: Integer;
	 const alpha: PSingle; (* host or device pointer *)
	 const A: PSingle; lda: Integer; const x: PSingle; incx: Integer; const beta: PSingle; (* host or device pointer *)
	 var y: Single; incy: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasSsymv_v2';

  function cublasDsymv_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; n: Integer;
	 const alpha: PDouble; (* host or device pointer *)
	 const A: PDouble; lda: Integer; const x: PDouble; incx: Integer; const beta: PDouble; (* host or device pointer *)
	 var y: double; incy: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasDsymv_v2';

  {function cublasChemv_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; n: Integer;
	 const alpha: p_cuComplex; (* host or device pointer *)
	 const A: p_cuComplex; lda: Integer; const x: p_cuComplex; incx: Integer; const beta: p_cuComplex; (* host or device pointer *)
	 var y: cuComplex; incy: Integer): cublasStatus_t; stdcall;

  function cublasZhemv_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; n: Integer; 
	 const alpha: p_cuDoubleComplex; (* host or device pointer *)
	 const A: p_cuDoubleComplex; lda: Integer; const x: p_cuDoubleComplex; incx: Integer; 
	 const beta: p_cuDoubleComplex; (* host or device pointer *)
	 var y: cuDoubleComplex; incy: Integer): cublasStatus_t; stdcall;    }

(* SBMV/HBMV *)
  function cublasSsbmv_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; n: Integer;
	 k: Integer; const alpha: PSingle; (* host or device pointer *)
	 const A: PSingle; lda: Integer; const x: PSingle; incx: Integer; const beta: PSingle; (* host or device pointer *)
	 var y: Single; incy: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasSsbmv_v2';

  function cublasDsbmv_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; n: Integer;
	 k: Integer; const alpha: PDouble; (* host or device pointer *)
	 const A: PDouble; lda: Integer; const x: PDouble; incx: Integer; const beta: PDouble; (* host or device pointer *)
	 var y: double; incy: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasDsbmv_v2';

  {function cublasChbmv_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; n: Integer;
	 k: Integer; const alpha: p_cuComplex; (* host or device pointer *)
	 const A: p_cuComplex; lda: Integer; const x: p_cuComplex; incx: Integer; const beta: p_cuComplex; (* host or device pointer *)
	 var y: cuComplex; incy: Integer): cublasStatus_t; stdcall;

  function cublasZhbmv_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; n: Integer; 
	 k: Integer; const alpha: p_cuDoubleComplex; (* host or device pointer *)
	 const A: p_cuDoubleComplex; lda: Integer; const x: p_cuDoubleComplex; incx: Integer; 
	 const beta: p_cuDoubleComplex; (* host or device pointer *)
	 var y: cuDoubleComplex; incy: Integer): cublasStatus_t; stdcall;             }

(* SPMV/HPMV *)
  function cublasSspmv_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; n: Integer;
	 const alpha: PSingle; (* host or device pointer *)
	 const AP: PSingle; const x: PSingle; incx: Integer; const beta: PSingle; (* host or device pointer *)
	 var y: Single; incy: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasSspmv_v2';

  function cublasDspmv_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; n: Integer;
	 const alpha: PDouble; (* host or device pointer *)
	 const AP: PDouble; const x: PDouble; incx: Integer; const beta: PDouble; (* host or device pointer *)
	 var y: double; incy: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasDspmv_v2';

  {function cublasChpmv_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; n: Integer;
	 const alpha: p_cuComplex; (* host or device pointer *)
	 const AP: p_cuComplex; const x: p_cuComplex; incx: Integer; const beta: p_cuComplex; (* host or device pointer *)
	 var y: cuComplex; incy: Integer): cublasStatus_t; stdcall;

  function cublasZhpmv_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; n: Integer; 
	 const alpha: p_cuDoubleComplex; (* host or device pointer *)
	 const AP: p_cuDoubleComplex; const x: p_cuDoubleComplex; incx: Integer; const beta: p_cuDoubleComplex; (* host or device pointer *)
	 var y: cuDoubleComplex; incy: Integer): cublasStatus_t; stdcall;        }

(* GER *)
  function cublasSger_v2(handle: cublasHandle_t; m: Integer; n: Integer; const alpha: PSingle; (* host or device pointer *)
	 const x: PSingle; incx: Integer; const y: PSingle; incy: Integer; var A: Single; 
	 lda: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasSger_v2';

  function cublasDger_v2(handle: cublasHandle_t; m: Integer; n: Integer; const alpha: PDouble; (* host or device pointer *)
	 const x: PDouble; incx: Integer; const y: PDouble; incy: Integer; var A: double; 
	 lda: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasDger_v2';

  {function cublasCgeru_v2(handle: cublasHandle_t; m: Integer; n: Integer; const alpha: p_cuComplex; (* host or device pointer *)
	 const x: p_cuComplex; incx: Integer; const y: p_cuComplex; incy: Integer; var A: cuComplex; 
	 lda: Integer): cublasStatus_t; stdcall;

  function cublasCgerc_v2(handle: cublasHandle_t; m: Integer; n: Integer; const alpha: p_cuComplex; (* host or device pointer *)
	 const x: p_cuComplex; incx: Integer; const y: p_cuComplex; incy: Integer; var A: cuComplex;
	 lda: Integer): cublasStatus_t; stdcall;

  function cublasZgeru_v2(handle: cublasHandle_t; m: Integer; n: Integer; const alpha: p_cuDoubleComplex; (* host or device pointer *)
	 const x: p_cuDoubleComplex; incx: Integer; const y: p_cuDoubleComplex; incy: Integer; 
	 var A: cuDoubleComplex; lda: Integer): cublasStatus_t; stdcall;

  function cublasZgerc_v2(handle: cublasHandle_t; m: Integer; n: Integer; const alpha: p_cuDoubleComplex; (* host or device pointer *)
	 const x: p_cuDoubleComplex; incx: Integer; const y: p_cuDoubleComplex; incy: Integer; 
	 var A: cuDoubleComplex; lda: Integer): cublasStatus_t; stdcall;      }

(* SYR/HER *)
  function cublasSsyr_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; n: Integer;
	 const alpha: PSingle; (* host or device pointer *)
	 const x: PSingle; incx: Integer; var A: Single; lda: Integer): cublasStatus_t;
   stdcall; external cublas_v2dll name 'cublasSsyr_v2';

  function cublasDsyr_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; n: Integer;
	 const alpha: PDouble; (* host or device pointer *)
	 const x: PDouble; incx: Integer; var A: double; lda: Integer): cublasStatus_t;
   stdcall; external cublas_v2dll name 'cublasDsyr_v2';

  {function cublasCher_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; n: Integer;
	 const alpha: PSingle; (* host or device pointer *)
	 const x: p_cuComplex; incx: Integer; var A: cuComplex; lda: Integer): cublasStatus_t; stdcall;

  function cublasZher_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; n: Integer; 
	 const alpha: PDouble; (* host or device pointer *)
	 const x: p_cuDoubleComplex; incx: Integer; var A: cuDoubleComplex; lda: Integer): cublasStatus_t; stdcall;    }

(* SPR/HPR *)
  function cublasSspr_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; n: Integer;
	 const alpha: PSingle; (* host or device pointer *)
	 const x: PSingle; incx: Integer; var AP: Single): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasSspr_v2';

  function cublasDspr_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; n: Integer;
	 const alpha: PDouble; (* host or device pointer *)
	 const x: PDouble; incx: Integer; var AP: double): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasDspr_v2';

  {function cublasChpr_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; n: Integer;
	 const alpha: PSingle; (* host or device pointer *)
	 const x: p_cuComplex; incx: Integer; var AP: cuComplex): cublasStatus_t; stdcall;

  function cublasZhpr_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; n: Integer; 
	 const alpha: PDouble; (* host or device pointer *)
	 const x: p_cuDoubleComplex; incx: Integer; var AP: cuDoubleComplex): cublasStatus_t; stdcall;   }

(* SYR2/HER2 *)
  function cublasSsyr2_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; n: Integer;
	 const alpha: PSingle; (* host or device pointer *)
	 const x: PSingle; incx: Integer; const y: PSingle; incy: Integer; var A: Single; 
	 lda: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasSsyr2_v2';

  function cublasDsyr2_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; n: Integer;
	 const alpha: PDouble; (* host or device pointer *)
	 const x: PDouble; incx: Integer; const y: PDouble; incy: Integer; var A: double; 
	 lda: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasDsyr2_v2';

  {function cublasCher2_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; n: Integer;
	 const alpha: p_cuComplex; (* host or device pointer *)
	 const x: p_cuComplex; incx: Integer; const y: p_cuComplex; incy: Integer; var A: cuComplex; 
	 lda: Integer): cublasStatus_t; stdcall;

  function cublasZher2_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; n: Integer; 
	 const alpha: p_cuDoubleComplex; (* host or device pointer *)
	 const x: p_cuDoubleComplex; incx: Integer; const y: p_cuDoubleComplex; incy: Integer; 
	 var A: cuDoubleComplex; lda: Integer): cublasStatus_t; stdcall;         }

(* SPR2/HPR2 *)
  function cublasSspr2_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; n: Integer;
	 const alpha: PSingle; (* host or device pointer *)
	 const x: PSingle; incx: Integer; const y: PSingle; incy: Integer; var AP: Single): cublasStatus_t;
   stdcall; external cublas_v2dll name 'cublasSspr2_v2';

  function cublasDspr2_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; n: Integer;
	 const alpha: PDouble; (* host or device pointer *)
	 const x: PDouble; incx: Integer; const y: PDouble; incy: Integer; var AP: double): cublasStatus_t;
   stdcall; external cublas_v2dll name 'cublasDspr2_v2';


  {function cublasChpr2_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; n: Integer;
	 const alpha: p_cuComplex; (* host or device pointer *)
	 const x: p_cuComplex; incx: Integer; const y: p_cuComplex; incy: Integer; var AP: cuComplex): cublasStatus_t; stdcall;

  function cublasZhpr2_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; n: Integer; 
	 const alpha: p_cuDoubleComplex; (* host or device pointer *)
	 const x: p_cuDoubleComplex; incx: Integer; const y: p_cuDoubleComplex; incy: Integer; 
	 var AP: cuDoubleComplex): cublasStatus_t; stdcall;                 }

(* ---------------- CUBLAS BLAS3 functions ---------------- *)

(* GEMM *)
  function cublasSgemm_v2(handle: cublasHandle_t; transa: cublasOperation_t; transb: cublasOperation_t;
	 m: Integer; n: Integer; k: Integer; const alpha: PSingle; (* host or device pointer *)
	 const A: PSingle; lda: Integer; const B: PSingle; ldb: Integer; const beta: PSingle; (* host or device pointer *)
	 var C: Single; ldc: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasSgemm_v2';

  function cublasDgemm_v2(handle: cublasHandle_t; transa: cublasOperation_t; transb: cublasOperation_t;
	 m: Integer; n: Integer; k: Integer; const alpha: PDouble; (* host or device pointer *)
	 const A: PDouble; lda: Integer; const B: PDouble; ldb: Integer; const beta: PDouble; (* host or device pointer *)
	 var C: double; ldc: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasDgemm_v2';

  {function cublasCgemm_v2(handle: cublasHandle_t; transa: cublasOperation_t; transb: cublasOperation_t;
	 m: Integer; n: Integer; k: Integer; const alpha: p_cuComplex; (* host or device pointer *)
	 const A: p_cuComplex; lda: Integer; const B: p_cuComplex; ldb: Integer; const beta: p_cuComplex; (* host or device pointer *)
	 var C: cuComplex; ldc: Integer): cublasStatus_t; stdcall;

  function cublasZgemm_v2(handle: cublasHandle_t; transa: cublasOperation_t; transb: cublasOperation_t; 
	 m: Integer; n: Integer; k: Integer; const alpha: p_cuDoubleComplex; (* host or device pointer *)
	 const A: p_cuDoubleComplex; lda: Integer; const B: p_cuDoubleComplex; ldb: Integer; 
	 const beta: p_cuDoubleComplex; (* host or device pointer *)
	 var C: cuDoubleComplex; ldc: Integer): cublasStatus_t; stdcall;       }

(* SYRK *)
  function cublasSsyrk_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; trans: cublasOperation_t;
	 n: Integer; k: Integer; const alpha: PSingle; (* host or device pointer *)
	 const A: PSingle; lda: Integer; const beta: PSingle; (* host or device pointer *)
	 var C: Single; ldc: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasSsyrk_v2';

  function cublasDsyrk_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; trans: cublasOperation_t;
	 n: Integer; k: Integer; const alpha: PDouble; (* host or device pointer *)
	 const A: PDouble; lda: Integer; const beta: PDouble; (* host or device pointer *)
	 var C: double; ldc: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasDsyrk_v2';

  {function cublasCsyrk_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; trans: cublasOperation_t;
	 n: Integer; k: Integer; const alpha: p_cuComplex; (* host or device pointer *)
	 const A: p_cuComplex; lda: Integer; const beta: p_cuComplex; (* host or device pointer *)
	 var C: cuComplex; ldc: Integer): cublasStatus_t; stdcall;

  function cublasZsyrk_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; trans: cublasOperation_t; 
	 n: Integer; k: Integer; const alpha: p_cuDoubleComplex; (* host or device pointer *)
	 const A: p_cuDoubleComplex; lda: Integer; const beta: p_cuDoubleComplex; (* host or device pointer *)
	 var C: cuDoubleComplex; ldc: Integer): cublasStatus_t; stdcall;
(* HERK *)
  function cublasCherk_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; trans: cublasOperation_t; 
	 n: Integer; k: Integer; const alpha: PSingle; (* host or device pointer *)
	 const A: p_cuComplex; lda: Integer; const beta: PSingle; (* host or device pointer *)
	 var C: cuComplex; ldc: Integer): cublasStatus_t; stdcall;

  function cublasZherk_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; trans: cublasOperation_t; 
	 n: Integer; k: Integer; const alpha: PDouble; (* host or device pointer *)
	 const A: p_cuDoubleComplex; lda: Integer; const beta: PDouble; (* host or device pointer *)
	 var C: cuDoubleComplex; ldc: Integer): cublasStatus_t; stdcall;         }

(* SYR2K *)
  function cublasSsyr2k_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; trans: cublasOperation_t;
	 n: Integer; k: Integer; const alpha: PSingle; (* host or device pointer *)
	 const A: PSingle; lda: Integer; const B: PSingle; ldb: Integer; const beta: PSingle; (* host or device pointer *)
	 var C: Single; ldc: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasSsyr2k_v2';

  function cublasDsyr2k_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; trans: cublasOperation_t;
	 n: Integer; k: Integer; const alpha: PDouble; (* host or device pointer *)
	 const A: PDouble; lda: Integer; const B: PDouble; ldb: Integer; const beta: PDouble; (* host or device pointer *)
	 var C: double; ldc: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasDsyr2k_v2';

  {function cublasCsyr2k_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; trans: cublasOperation_t;
	 n: Integer; k: Integer; const alpha: p_cuComplex; (* host or device pointer *)
	 const A: p_cuComplex; lda: Integer; const B: p_cuComplex; ldb: Integer; const beta: p_cuComplex; (* host or device pointer *)
	 var C: cuComplex; ldc: Integer): cublasStatus_t; stdcall;

  function cublasZsyr2k_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; trans: cublasOperation_t; 
	 n: Integer; k: Integer; const alpha: p_cuDoubleComplex; (* host or device pointer *)
	 const A: p_cuDoubleComplex; lda: Integer; const B: p_cuDoubleComplex; ldb: Integer; 
	 const beta: p_cuDoubleComplex; (* host or device pointer *)
	 var C: cuDoubleComplex; ldc: Integer): cublasStatus_t; stdcall;
(* HER2K *)
  function cublasCher2k_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; trans: cublasOperation_t; 
	 n: Integer; k: Integer; const alpha: p_cuComplex; (* host or device pointer *)
	 const A: p_cuComplex; lda: Integer; const B: p_cuComplex; ldb: Integer; const beta: PSingle; (* host or device pointer *)
	 var C: cuComplex; ldc: Integer): cublasStatus_t; stdcall;

  function cublasZher2k_v2(handle: cublasHandle_t; uplo: cublasFillMode_t; trans: cublasOperation_t; 
	 n: Integer; k: Integer; const alpha: p_cuDoubleComplex; (* host or device pointer *)
	 const A: p_cuDoubleComplex; lda: Integer; const B: p_cuDoubleComplex; ldb: Integer; 
	 const beta: PDouble; (* host or device pointer *)
	 var C: cuDoubleComplex; ldc: Integer): cublasStatus_t; stdcall;            }

(* SYMM *)
  function cublasSsymm_v2(handle: cublasHandle_t; side: cublasSideMode_t; uplo: cublasFillMode_t;
	 m: Integer; n: Integer; const alpha: PSingle; (* host or device pointer *)
	 const A: PSingle; lda: Integer; const B: PSingle; ldb: Integer; const beta: PSingle; (* host or device pointer *)
	 var C: Single; ldc: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasSsymm_v2';

  function cublasDsymm_v2(handle: cublasHandle_t; side: cublasSideMode_t; uplo: cublasFillMode_t;
	 m: Integer; n: Integer; const alpha: PDouble; (* host or device pointer *)
	 const A: PDouble; lda: Integer; const B: PDouble; ldb: Integer; const beta: PDouble; (* host or device pointer *)
	 var C: double; ldc: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasDsymm_v2';

  {function cublasCsymm_v2(handle: cublasHandle_t; side: cublasSideMode_t; uplo: cublasFillMode_t;
	 m: Integer; n: Integer; const alpha: p_cuComplex; (* host or device pointer *)
	 const A: p_cuComplex; lda: Integer; const B: p_cuComplex; ldb: Integer; const beta: p_cuComplex; (* host or device pointer *)
	 var C: cuComplex; ldc: Integer): cublasStatus_t; stdcall;

  function cublasZsymm_v2(handle: cublasHandle_t; side: cublasSideMode_t; uplo: cublasFillMode_t; 
	 m: Integer; n: Integer; const alpha: p_cuDoubleComplex; (* host or device pointer *)
	 const A: p_cuDoubleComplex; lda: Integer; const B: p_cuDoubleComplex; ldb: Integer; 
	 const beta: p_cuDoubleComplex; (* host or device pointer *)
	 var C: cuDoubleComplex; ldc: Integer): cublasStatus_t; stdcall;

(* HEMM *)
  function cublasChemm_v2(handle: cublasHandle_t; side: cublasSideMode_t; uplo: cublasFillMode_t; 
	 m: Integer; n: Integer; const alpha: p_cuComplex; (* host or device pointer *)
	 const A: p_cuComplex; lda: Integer; const B: p_cuComplex; ldb: Integer; const beta: p_cuComplex; (* host or device pointer *)
	 var C: cuComplex; ldc: Integer): cublasStatus_t; stdcall;

  function cublasZhemm_v2(handle: cublasHandle_t; side: cublasSideMode_t; uplo: cublasFillMode_t; 
	 m: Integer; n: Integer; const alpha: p_cuDoubleComplex; (* host or device pointer *)
	 const A: p_cuDoubleComplex; lda: Integer; const B: p_cuDoubleComplex; ldb: Integer; 
	 const beta: p_cuDoubleComplex; (* host or device pointer *)
	 var C: cuDoubleComplex; ldc: Integer): cublasStatus_t; stdcall;         }

(* TRSM *)
  function cublasStrsm_v2(handle: cublasHandle_t; side: cublasSideMode_t; uplo: cublasFillMode_t;
	 trans: cublasOperation_t; diag: cublasDiagType_t; m: Integer; n: Integer; const alpha: PSingle; (* host or device pointer *)
	 const A: PSingle; lda: Integer; var B: Single; ldb: Integer): cublasStatus_t;
   stdcall; external cublas_v2dll name 'cublasStrsm_v2';


  function cublasDtrsm_v2(handle: cublasHandle_t; side: cublasSideMode_t; uplo: cublasFillMode_t;
	 trans: cublasOperation_t; diag: cublasDiagType_t; m: Integer; n: Integer; const alpha: PDouble; (* host or device pointer *)
	 const A: PDouble; lda: Integer; var B: double; ldb: Integer): cublasStatus_t;
   stdcall; external cublas_v2dll name 'cublasDtrsm_v2';

  {function cublasCtrsm_v2(handle: cublasHandle_t; side: cublasSideMode_t; uplo: cublasFillMode_t;
	 trans: cublasOperation_t; diag: cublasDiagType_t; m: Integer; n: Integer; const alpha: p_cuComplex; (* host or device pointer *)
	 const A: p_cuComplex; lda: Integer; var B: cuComplex; ldb: Integer): cublasStatus_t; stdcall;

  function cublasZtrsm_v2(handle: cublasHandle_t; side: cublasSideMode_t; uplo: cublasFillMode_t; 
	 trans: cublasOperation_t; diag: cublasDiagType_t; m: Integer; n: Integer; const alpha: p_cuDoubleComplex; (* host or device pointer *)
	 const A: p_cuDoubleComplex; lda: Integer; var B: cuDoubleComplex; ldb: Integer): cublasStatus_t; stdcall;       }

(* TRMM *)
  function cublasStrmm_v2(handle: cublasHandle_t; side: cublasSideMode_t; uplo: cublasFillMode_t;
	 trans: cublasOperation_t; diag: cublasDiagType_t; m: Integer; n: Integer; const alpha: PSingle; (* host or device pointer *)
	 const A: PSingle; lda: Integer; const B: PSingle; ldb: Integer; var C: Single; 
	 ldc: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasStrmm_v2';

  function cublasDtrmm_v2(handle: cublasHandle_t; side: cublasSideMode_t; uplo: cublasFillMode_t;
	 trans: cublasOperation_t; diag: cublasDiagType_t; m: Integer; n: Integer; const alpha: PDouble; (* host or device pointer *)
	 const A: PDouble; lda: Integer; const B: PDouble; ldb: Integer; var C: double; 
	 ldc: Integer): cublasStatus_t; stdcall; external cublas_v2dll name 'cublasDtrmm_v2';

  {function cublasCtrmm_v2(handle: cublasHandle_t; side: cublasSideMode_t; uplo: cublasFillMode_t;
	 trans: cublasOperation_t; diag: cublasDiagType_t; m: Integer; n: Integer; const alpha: p_cuComplex; (* host or device pointer *)
	 const A: p_cuComplex; lda: Integer; const B: p_cuComplex; ldb: Integer; var C: cuComplex; 
	 ldc: Integer): cublasStatus_t; stdcall;

  function cublasZtrmm_v2(handle: cublasHandle_t; side: cublasSideMode_t; uplo: cublasFillMode_t; 
	 trans: cublasOperation_t; diag: cublasDiagType_t; m: Integer; n: Integer; const alpha: p_cuDoubleComplex; (* host or device pointer *)
	 const A: p_cuDoubleComplex; lda: Integer; const B: p_cuDoubleComplex; ldb: Integer; 
	 var C: cuDoubleComplex; ldc: Integer): cublasStatus_t; stdcall;       }


implementation

end.
