unit MNA;

interface

uses Math387, CuBLASMtxVec, Classes;

type
   TMNA = Class;

   TElement=Class
   protected
   n_1, n_2, n_3, n_4, n_5, //Nodes
   m_1, m_2: integer;       //Extra
   Ckt: TMNA;
   procedure SetCkt(Ckto: TMNA);
   public
   function max_node: integer;
   function nonlinear: Boolean;
   procedure KCL(n1, n2: integer; Current: TSample);
   function KVL(n1, n2: integer; Voltage: Tsample; x: TVec): Tsample;
   procedure Eval(t: TSample; x, dxdt: TVec); virtual; abstract;
   end;


   TMNA=Class
   private
   num_nodes, num_vsources, num_nodess, inc_num_equations,
   num_inductors, num_opamps, num_equations: Integer;
   first_pass, nonlinear: Boolean;
   Jac: TMtx;
   MNE: TVec;
   Cir: TList;
   procedure KCL(n_1, n_2: integer; Current: TSample);
   function KVL(n_1, n_2: integer; Voltage: Tsample; x: TVec): TSample;
   public
   constructor Create;
   function getSize: integer;
   function getNum_Nodes: integer;
   function getNonlinearity: Boolean;
   procedure setNonlinearity(no_linear: boolean);
   procedure Add(Component: TElement);
   procedure DaeFunction(t:TSample; x, dxdt, Result:TVec);
   procedure Jacobian(t: TSample; x, dxdt:TVec; Jac: TMtx);
   function DCAnalysis: TVec;
   function DampedNewton: TVec;
   destructor Destroy; override;
   end;

implementation

uses Math, ComponentsMNA;

{ TElement }

procedure TElement.KCL(n1, n2: integer; Current: TSample);
begin
  Ckt.KCL(n1,n2,Current);
end;

function TElement.KVL(n1, n2: integer; Voltage: Tsample; x: TVec): Tsample;
begin
  result:= Ckt.KVL(n1,n2,Voltage,x);
end;

function TElement.max_node: integer;
begin
  result:= Math.Max(n_1, n_2);
  result:= Math.Max(result,n_3);
  result:= Math.Max(result,n_4);
  result:= Math.Max(result,n_5);
end;

function TElement.nonlinear: Boolean;
begin
  result:= False;
end;

procedure TElement.Setckt(Ckto: TMNA);
begin
  Ckt:= Ckto;
end;

{ TMNA }

constructor TMNA.Create;
begin
  Cir:= TList.Create;
  MNE:= TVec.Create;
  nonlinear:= false;
end;

procedure TMNA.KCL(n_1, n_2: integer;Current: TSample);
begin
  if (n_1 <> Datum) then
       MNE[n_1]:= MNE[n_1]-Current;
  if (n_2 <> Datum) then
       MNE[n_2]:= MNE[n_2]+Current;
end;

function TMNA.KVL(n_1, n_2: integer;Voltage: Tsample; x: TVec): TSample;
begin
  inc(inc_num_equations);
   if (n_1 <> Datum) then
        begin
           MNE[n_1]:= MNE[n_1]-x[num_nodes+inc_num_equations];
           MNE[num_nodes+inc_num_equations]:= x[n_1];
        end;
   if (n_2 <> Datum) then
        begin
           MNE[n_2]:= MNE[n_2]+x[num_nodes+inc_num_equations];
           MNE[num_nodes+inc_num_equations]:= MNE[num_nodes+inc_num_equations]-x[n_2];
        end;
   MNE[num_nodes+inc_num_equations]:= MNE[num_nodes+inc_num_equations]-Voltage;
   Result:= x[num_nodes+inc_num_equations];
end;

procedure TMNA.DaeFunction(t:TSample;x,dxdt,Result:TVec);
var i: integer;
begin
  MNE.SetZero;
  inc_num_equations:= 0;
  i:= 0;
  while (i < Cir.Count) do
  begin
    TElement(Cir[i]).Eval(t,x,dxdt);
    inc(i);
  end; // end while
  Result.Assign(MNE);
end;

procedure TMNA.Add(Component: TElement);
begin
   Cir.Add(Component);
   Component.SetCkt(Self);
   //num_nodes:= 0;
   if (TElement(Component) is TVSinSource) then
    begin
      num_nodes:= math.Max(num_nodes, Component.max_node);
      inc(num_vsources);
    end;
   if (TElement(Component) is TVABSSinSource) then
    begin
      num_nodes:= math.Max(num_nodes, Component.max_node);
      inc(num_vsources);
    end;
   if (TElement(Component) is TDCVSource) then
    begin
      num_nodes:= math.Max(num_nodes, Component.max_node);
      inc(num_vsources);
    end;
   if (TElement(Component) is TDCISource)then
    begin
      num_nodes:= math.Max(num_nodes, Component.max_node);
    end;
   if (TElement(Component) is TDiode) then
    begin
      nonlinear:= true;
      num_nodes:= math.Max(num_nodes, Component.max_node);
    end;
   if (TElement(Component) is TDiodeZener) then
    begin
      nonlinear:= true;
      num_nodes:= math.Max(num_nodes, Component.max_node);
    end;
   if (TElement(Component) is TDiodeModelica) then
    begin
      nonlinear:= true;
      num_nodes:= math.Max(num_nodes, Component.max_node);
    end;
   if (TElement(Component) is TResistor) then
    begin
      num_nodes:= math.Max(num_nodes, Component.max_node);
    end;
   if (TElement(Component) is TChuaResistor) then
    begin
      nonlinear:= true;
      num_nodes:= math.Max(num_nodes, Component.max_node);
    end;
   if (TElement(Component) is TCapacitor) then
    begin
      num_nodes:= math.Max(num_nodes, Component.max_node);
    end;
   if (TElement(Component) is TInductor) then
    begin
      num_nodes:= math.Max(num_nodes, Component.max_node);
      inc(num_inductors);
    end;
   if (TElement(Component) is TOPAMPIDEAL) then
    begin
      num_nodes:= math.Max(num_nodes, Component.max_node);
      inc(num_vsources);
    end;
   if (TElement(Component) is TOPAMP) then
    begin
      nonlinear:= true;
      num_nodes:= math.Max(num_nodes, Component.max_node);
      inc(num_vsources);
    end;
   if (TElement(Component) is TOPAMP741) then
    begin
      nonlinear:= true;
      num_nodes:= math.Max(num_nodes, Component.max_node);
      inc(num_opamps,21);
    end;
   if (TElement(Component) is TVCCS) then
    begin
       num_nodes:= math.Max(num_nodes, Component.max_node);
    end;
   if (TElement(Component) is TVCVS) then
    begin
       num_nodes:= math.Max(num_nodes, Component.max_node);
       inc(num_vsources);
    end;
    if (TElement(Component) is TNorton) then
    begin
       num_nodes:= math.Max(num_nodes, Component.max_node);
       inc(num_vsources);
    end;
    if (TElement(Component) is TLinearBJTNPN) then
    begin
      num_nodes:= math.Max(num_nodes, Component.max_node);
      inc(num_vsources);
    end;
    if (TElement(Component) is TLinearBJTPNP) then
    begin
      num_nodes:= math.Max(num_nodes, Component.max_node);
      inc(num_vsources);
    end;
    if (TElement(Component) is TBJTNPN) then
    begin
      nonlinear:= true;
      num_nodes:= math.Max(num_nodes, Component.max_node);
    end;
   if (TElement(Component) is TBJTEMNPN) then
    begin
      nonlinear:= true;
      num_nodes:= math.Max(num_nodes, Component.max_node);
    end;
   if (TElement(Component) is TBJTPWLNPN) then
    begin
      nonlinear:= true;
      num_nodes:= math.Max(num_nodes, Component.max_node);
    end;
   if (TElement(Component) is TBJTPWLPNP) then
    begin
      nonlinear:= true;
      num_nodes:= math.Max(num_nodes, Component.max_node);
    end;
   if (TElement(Component) is TBJTEMPNP) then
    begin
      nonlinear:= true;
      num_nodes:= math.Max(num_nodes, Component.max_node);
    end;
   if (TElement(Component) is TBJTNPNF) then
    begin
      nonlinear:= true;
      num_nodes:= math.Max(num_nodes, Component.max_node);
    end;
    if (TElement(Component) is TBJTPNP) then
    begin
      nonlinear:= true;
      num_nodes:= math.Max(num_nodes, Component.max_node);
    end;
   if (TElement(Component) is TBJTPNPF) then
    begin
      nonlinear:= true;
      num_nodes:= math.Max(num_nodes, Component.max_node);
    end;
  num_equations:=(num_nodes+1)+num_vsources+num_inductors;
  MNE.Resize(num_equations);
  first_pass:= True;
end;

destructor TMNA.Destroy;
var i: integer;
begin
  for i:=Cir.Count-1 downto 0 do
    Cir.Delete(i);
  Cir.Destroy;
  MNE.Destroy;
  inherited;
end;

function TMNA.getSize: integer;
begin
  num_nodess:= num_nodes+1;
  first_pass:= False;
  result:= num_equations;
end;

function TMNA.getNum_Nodes: integer;
begin
  getsize;
  result:= num_nodess;
end;

function TMNA.getNonlinearity: Boolean;
begin
  result:= nonlinear;
end;

procedure TMNA.setNonlinearity(no_linear: boolean);
begin
  nonlinear:= no_linear;
end;

procedure TMNA.Jacobian(t: TSample; x, dxdt: TVec; Jac: TMtx);
var
   i:integer;
   temp, fun, funi: TVec;
begin
 temp:= TVec.Create;
 temp.Size(num_equations);
 fun:= TVec.Create;
 fun.Size(num_equations);
 funi:= TVec.Create;
 funi.Size(num_equations);
 DaeFunction(t,x,dxdt,fun);
 for i:=0 to num_equations-1 do
  begin
    temp.Assign(x);
    temp[i]:=temp[i]+1e-7;
    temp.Add(x);
    DaeFunction(t,temp,dxdt,funi); //F(t,y+diei,alfa*(y+diei)+beta)
    funi.Sub(fun);                 //fun1:= F(t,y,alfa*y+beta)
    funi.Scale(1/1e-7);
    Jac.SetCol(funi,i);
  end;
end;

function TMNA.DCAnalysis: TVec;
var
  t, sol: double;
  x, dxdt, B: TVec;
begin
  t:= 0;
  x:= TVec.Create;
  x.Size(num_equations);
  dxdt:= TVec.Create;
  dxdt.Size(num_equations);
  B:= TVec.Create;
  B.Size(num_equations);
  Jac:= TMtx.Create;
  Jac.Size(num_equations,num_equations);
  Jacobian(t, x, dxdt, Jac);
  DaeFunction(t,x,dxdt,B);
  B.Scale(-1);
  result:= TVec(x);
  Jac.LUSolve(B, X);
  {sol:= Jac.AdaptiveSGMRES(B, X);
  if (sol > 70 ) then
    sol:= Jac.AdaptiveSGMRES(B, X);
  if ( sol > 70 ) then
    sol:= Jac.BiCGStab(B, X);              // Improve Solution  }
end;

function TMNA.DampedNewton: TVec;
var
  t, funnorm: double;
  x, dxdt, fun: TVec;
begin
  t:= 0;
  x:= TVec.Create;
  x.Size(num_equations);
  dxdt:= TVec.Create;
  dxdt.Size(num_equations);
  fun:= TVec.Create;
  fun.Size(num_equations);
  result:= TVec.Create;
  result.Size(num_equations);
  result.Assign(DCAnalysis);
  funnorm:= fun.NormL2;
end;

end.
